@@ -1,3 +1,4 @@
+/* -*- Mode: c; tab-width: 8; c-basic-offset: 4; indent-tabs-mode: t; -*- */
 /* cairo - a vector graphics library with display and print output
  *
  * Copyright © 2002 University of Southern California
@@ -53,6 +54,8 @@ _cairo_clip_init (cairo_clip_t *clip, cairo_surface_t *target)
     else
 	clip->mode = CAIRO_CLIP_MODE_MASK;
 
+    clip->all_clipped = FALSE;
+
     clip->surface = NULL;
     clip->surface_rect.x = 0;
     clip->surface_rect.y = 0;
@@ -61,75 +64,88 @@ _cairo_clip_init (cairo_clip_t *clip, cairo_surface_t *target)
 
     clip->serial = 0;
 
-    clip->region = NULL;
+    _cairo_region_init (&clip->region);
+    clip->has_region = FALSE;
 
     clip->path = NULL;
 }
 
-void
-_cairo_clip_fini (cairo_clip_t *clip)
-{
-    cairo_surface_destroy (clip->surface);
-    clip->surface = NULL;
-
-    clip->serial = 0;
-
-    if (clip->region)
-	pixman_region_destroy (clip->region);
-    clip->region = NULL;
-
-    _cairo_clip_path_destroy (clip->path);
-    clip->path = NULL;
-}
-
-void
+cairo_status_t
 _cairo_clip_init_copy (cairo_clip_t *clip, cairo_clip_t *other)
 {
     clip->mode = other->mode;
 
+    clip->all_clipped = other->all_clipped;
+
     clip->surface = cairo_surface_reference (other->surface);
     clip->surface_rect = other->surface_rect;
 
     clip->serial = other->serial;
 
-    if (other->region == NULL) {
-	clip->region = other->region;
+    _cairo_region_init (&clip->region);
+
+    if (other->has_region) {
+	cairo_status_t status;
+	status = _cairo_region_copy (&clip->region, &other->region);
+	if (status) {
+	    _cairo_region_fini (&clip->region);
+	    cairo_surface_destroy (clip->surface);
+	    return status;
+	}
+        clip->has_region = TRUE;
     } else {
-	clip->region = pixman_region_create ();
-	pixman_region_copy (clip->region, other->region);
+        clip->has_region = FALSE;
     }
 
     clip->path = _cairo_clip_path_reference (other->path);
+
+    return CAIRO_STATUS_SUCCESS;
 }
 
-cairo_status_t
+void
 _cairo_clip_reset (cairo_clip_t *clip)
 {
+    clip->all_clipped = FALSE;
+
     /* destroy any existing clip-region artifacts */
     cairo_surface_destroy (clip->surface);
     clip->surface = NULL;
 
     clip->serial = 0;
 
-    if (clip->region)
-	pixman_region_destroy (clip->region);
-    clip->region = NULL;
+    if (clip->has_region) {
+        /* _cairo_region_fini just releases the resources used but
+         * doesn't bother with leaving the region in a valid state.
+         * So _cairo_region_init has to be called afterwards. */
+	_cairo_region_fini (&clip->region);
+        _cairo_region_init (&clip->region);
+
+        clip->has_region = FALSE;
+    }
 
     _cairo_clip_path_destroy (clip->path);
     clip->path = NULL;
-
-    return CAIRO_STATUS_SUCCESS;
 }
 
+static void
+_cairo_clip_set_all_clipped (cairo_clip_t *clip, cairo_surface_t *target)
+{
+    _cairo_clip_reset (clip);
+
+    clip->all_clipped = TRUE;
+    clip->serial = _cairo_surface_allocate_clip_serial (target);
+}
+
+
 static cairo_status_t
 _cairo_clip_path_intersect_to_rectangle (cairo_clip_path_t       *clip_path,
-   				         cairo_rectangle_int16_t *rectangle)
+				         cairo_rectangle_int_t   *rectangle)
 {
     while (clip_path) {
         cairo_status_t status;
         cairo_traps_t traps;
         cairo_box_t extents;
-        cairo_rectangle_int16_t extents_rect;
+        cairo_rectangle_int_t extents_rect;
 
         _cairo_traps_init (&traps);
 
@@ -156,41 +172,41 @@ _cairo_clip_path_intersect_to_rectangle (cairo_clip_path_t       *clip_path,
 
 cairo_status_t
 _cairo_clip_intersect_to_rectangle (cairo_clip_t            *clip,
-				    cairo_rectangle_int16_t *rectangle)
+				    cairo_rectangle_int_t *rectangle)
 {
     if (!clip)
 	return CAIRO_STATUS_SUCCESS;
 
+    if (clip->all_clipped) {
+	*rectangle = clip->surface_rect;
+	return CAIRO_STATUS_SUCCESS;
+    }
+
     if (clip->path) {
         cairo_status_t status;
-        
+
         status = _cairo_clip_path_intersect_to_rectangle (clip->path,
                                                           rectangle);
         if (status)
             return status;
     }
 
-    if (clip->region) {
-	pixman_region16_t *intersection;
+    if (clip->has_region) {
 	cairo_status_t status = CAIRO_STATUS_SUCCESS;
-	pixman_region_status_t pixman_status;
+	cairo_region_t intersection;
 
-	intersection = _cairo_region_create_from_rectangle (rectangle);
-	if (intersection == NULL)
-	    return CAIRO_STATUS_NO_MEMORY;
+	_cairo_region_init_rect (&intersection, rectangle);
 
-	pixman_status = pixman_region_intersect (intersection,
-					  clip->region,
-					  intersection);
-	if (pixman_status == PIXMAN_REGION_STATUS_SUCCESS)
-	    _cairo_region_extents_rectangle (intersection, rectangle);
-	else
-	    status = CAIRO_STATUS_NO_MEMORY;
+	status = _cairo_region_intersect (&intersection, &clip->region,
+					  &intersection);
 
-	pixman_region_destroy (intersection);
+	if (!status)
+	    _cairo_region_get_extents (&intersection, rectangle);
 
-	if (status)
-	    return status;
+        _cairo_region_fini (&intersection);
+
+        if (status)
+            return status;
     }
 
     if (clip->surface)
@@ -201,39 +217,48 @@ _cairo_clip_intersect_to_rectangle (cairo_clip_t            *clip,
 
 cairo_status_t
 _cairo_clip_intersect_to_region (cairo_clip_t      *clip,
-				 pixman_region16_t *region)
+				 cairo_region_t *region)
 {
+    cairo_status_t status;
+
     if (!clip)
 	return CAIRO_STATUS_SUCCESS;
 
+    if (clip->all_clipped) {
+	cairo_region_t clip_rect;
+
+	_cairo_region_init_rect (&clip_rect, &clip->surface_rect);
+
+	status = _cairo_region_intersect (region, &clip_rect, region);
+
+	_cairo_region_fini (&clip_rect);
+
+	return status;
+    }
+
     if (clip->path) {
 	/* Intersect clip path into region. */
     }
 
-    if (clip->region)
-	pixman_region_intersect (region, clip->region, region);
-
-    if (clip->surface) {
-	pixman_region16_t *clip_rect;
-	pixman_region_status_t pixman_status;
-	cairo_status_t status = CAIRO_STATUS_SUCCESS;
-
-	clip_rect = _cairo_region_create_from_rectangle (&clip->surface_rect);
-	if (clip_rect == NULL)
-	    return CAIRO_STATUS_NO_MEMORY;
-
-	pixman_status = pixman_region_intersect (region,
-						 clip_rect,
-						 region);
-	if (pixman_status != PIXMAN_REGION_STATUS_SUCCESS)
-	    status = CAIRO_STATUS_NO_MEMORY;
-
-	pixman_region_destroy (clip_rect);
-
+    if (clip->has_region) {
+	status = _cairo_region_intersect (region, &clip->region, region);
 	if (status)
 	    return status;
     }
 
+    if (clip->surface) {
+	cairo_region_t clip_rect;
+
+	_cairo_region_init_rect (&clip_rect, &clip->surface_rect);
+
+	status = _cairo_region_intersect (region, &clip_rect, region);
+
+	_cairo_region_fini (&clip_rect);
+
+        if (status)
+            return status;
+    }
+
     return CAIRO_STATUS_SUCCESS;
 }
 
@@ -247,11 +272,14 @@ _cairo_clip_combine_to_surface (cairo_clip_t                  *clip,
 				cairo_surface_t               *dst,
 				int                           dst_x,
 				int                           dst_y,
-				const cairo_rectangle_int16_t *extents)
+				const cairo_rectangle_int_t *extents)
 {
     cairo_pattern_union_t pattern;
     cairo_status_t status;
 
+    if (clip->all_clipped)
+	return CAIRO_STATUS_SUCCESS;
+
     _cairo_pattern_init_for_surface (&pattern.surface, clip->surface);
 
     status = _cairo_surface_composite (op,
@@ -285,7 +313,7 @@ _cairo_clip_intersect_path (cairo_clip_t       *clip,
 
     clip_path = malloc (sizeof (cairo_clip_path_t));
     if (clip_path == NULL)
-	return CAIRO_STATUS_NO_MEMORY;
+	return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 
     status = _cairo_path_fixed_init_copy (&clip_path->path, path);
     if (status) {
@@ -293,7 +321,7 @@ _cairo_clip_intersect_path (cairo_clip_t       *clip,
 	return status;
     }
 
-    clip_path->ref_count = 1;
+    CAIRO_REFERENCE_COUNT_INIT (&clip_path->ref_count, 1);
     clip_path->fill_rule = fill_rule;
     clip_path->tolerance = tolerance;
     clip_path->antialias = antialias;
@@ -309,7 +337,9 @@ _cairo_clip_path_reference (cairo_clip_path_t *clip_path)
     if (clip_path == NULL)
 	return NULL;
 
-    clip_path->ref_count++;
+    assert (CAIRO_REFERENCE_COUNT_HAS_REFERENCE (&clip_path->ref_count));
+
+    _cairo_reference_count_inc (&clip_path->ref_count);
 
     return clip_path;
 }
@@ -320,8 +350,9 @@ _cairo_clip_path_destroy (cairo_clip_path_t *clip_path)
     if (clip_path == NULL)
 	return;
 
-    clip_path->ref_count--;
-    if (clip_path->ref_count)
+    assert (CAIRO_REFERENCE_COUNT_HAS_REFERENCE (&clip_path->ref_count));
+
+    if (! _cairo_reference_count_dec_and_test (&clip_path->ref_count))
 	return;
 
     _cairo_path_fixed_fini (&clip_path->path);
@@ -329,42 +360,51 @@ _cairo_clip_path_destroy (cairo_clip_path_t *clip_path)
     free (clip_path);
 }
 
-static cairo_status_t
+
+static cairo_int_status_t
 _cairo_clip_intersect_region (cairo_clip_t    *clip,
 			      cairo_traps_t   *traps,
 			      cairo_surface_t *target)
 {
-    pixman_region16_t *region;
-    cairo_status_t status;
+    cairo_region_t region;
+    cairo_int_status_t status;
+
+    if (clip->all_clipped)
+	return CAIRO_STATUS_SUCCESS;
 
     if (clip->mode != CAIRO_CLIP_MODE_REGION)
 	return CAIRO_INT_STATUS_UNSUPPORTED;
 
     status = _cairo_traps_extract_region (traps, &region);
+
     if (status)
 	return status;
 
-    if (region == NULL)
-	return CAIRO_INT_STATUS_UNSUPPORTED;
-
     status = CAIRO_STATUS_SUCCESS;
-    if (clip->region == NULL) {
-	clip->region = region;
-    } else {
-	pixman_region16_t *intersection = pixman_region_create();
 
-	if (pixman_region_intersect (intersection,
-				     clip->region, region)
-	    == PIXMAN_REGION_STATUS_SUCCESS) {
-	    pixman_region_destroy (clip->region);
-	    clip->region = intersection;
-	} else {
-	    status = CAIRO_STATUS_NO_MEMORY;
-	}
-	pixman_region_destroy (region);
+    if (!clip->has_region) {
+        status = _cairo_region_copy (&clip->region, &region);
+	if (status == CAIRO_STATUS_SUCCESS)
+	    clip->has_region = TRUE;
+    } else {
+	cairo_region_t intersection;
+        _cairo_region_init (&intersection);
+
+	status = _cairo_region_intersect (&intersection,
+		                         &clip->region,
+		                         &region);
+
+	if (status == CAIRO_STATUS_SUCCESS)
+	    status = _cairo_region_copy (&clip->region, &intersection);
+
+        _cairo_region_fini (&intersection);
     }
 
     clip->serial = _cairo_surface_allocate_clip_serial (target);
+    _cairo_region_fini (&region);
+
+    if (! _cairo_region_not_empty (&clip->region))
+	_cairo_clip_set_all_clipped (clip, target);
 
     return status;
 }
@@ -377,10 +417,13 @@ _cairo_clip_intersect_mask (cairo_clip_t      *clip,
 {
     cairo_pattern_union_t pattern;
     cairo_box_t extents;
-    cairo_rectangle_int16_t surface_rect, target_rect;
+    cairo_rectangle_int_t surface_rect, target_rect;
     cairo_surface_t *surface;
     cairo_status_t status;
 
+    if (clip->all_clipped)
+	return CAIRO_STATUS_SUCCESS;
+
     /* Represent the clip as a mask surface.  We create a new surface
      * the size of the intersection of the old mask surface and the
      * extents of the new clip path. */
@@ -398,18 +441,30 @@ _cairo_clip_intersect_mask (cairo_clip_t      *clip,
     if (!status)
 	_cairo_rectangle_intersect (&surface_rect, &target_rect);
 
+    if (surface_rect.width == 0 || surface_rect.height == 0) {
+	surface = NULL;
+	status = CAIRO_STATUS_SUCCESS;
+	if (clip->surface != NULL)
+	    cairo_surface_destroy (clip->surface);
+	goto DONE;
+    }
+
+    _cairo_pattern_init_solid (&pattern.solid, CAIRO_COLOR_WHITE,
+			       CAIRO_CONTENT_COLOR);
     surface = _cairo_surface_create_similar_solid (target,
 						   CAIRO_CONTENT_ALPHA,
 						   surface_rect.width,
 						   surface_rect.height,
-						   CAIRO_COLOR_WHITE);
-    if (surface->status)
-	return CAIRO_STATUS_NO_MEMORY;
+						   CAIRO_COLOR_WHITE,
+						   &pattern.base);
+    if (surface->status) {
+	_cairo_pattern_fini (&pattern.base);
+	return surface->status;
+    }
 
     /* Render the new clipping path into the new mask surface. */
 
     _cairo_traps_translate (traps, -surface_rect.x, -surface_rect.y);
-    _cairo_pattern_init_solid (&pattern.solid, CAIRO_COLOR_WHITE);
 
     status = _cairo_surface_composite_trapezoids (CAIRO_OPERATOR_IN,
 						  &pattern.base,
@@ -457,10 +512,14 @@ _cairo_clip_intersect_mask (cairo_clip_t      *clip,
 	cairo_surface_destroy (clip->surface);
     }
 
+ DONE:
     clip->surface = surface;
     clip->surface_rect = surface_rect;
     clip->serial = _cairo_surface_allocate_clip_serial (target);
 
+    if (surface_rect.width == 0 || surface_rect.height == 0)
+	_cairo_clip_set_all_clipped (clip, target);
+
     return status;
 }
 
@@ -475,6 +534,15 @@ _cairo_clip_clip (cairo_clip_t       *clip,
     cairo_status_t status;
     cairo_traps_t traps;
 
+    if (clip->all_clipped)
+	return CAIRO_STATUS_SUCCESS;
+
+    /* catch the empty clip path */
+    if (! path->has_current_point) {
+	_cairo_clip_set_all_clipped (clip, target);
+	return CAIRO_STATUS_SUCCESS;
+    }
+
     status = _cairo_clip_intersect_path (clip,
 					 path, fill_rule, tolerance,
 					 antialias);
@@ -509,8 +577,11 @@ _cairo_clip_translate (cairo_clip_t  *clip,
                        cairo_fixed_t  tx,
                        cairo_fixed_t  ty)
 {
-    if (clip->region) {
-        pixman_region_translate (clip->region,
+    if (clip->all_clipped)
+	return;
+
+    if (clip->has_region) {
+        _cairo_region_translate (&clip->region,
                                  _cairo_fixed_integer_part (tx),
                                  _cairo_fixed_integer_part (ty));
     }
@@ -535,50 +606,75 @@ _cairo_clip_translate (cairo_clip_t  *clip,
     }
 }
 
-static void
+static cairo_status_t
 _cairo_clip_path_reapply_clip_path (cairo_clip_t      *clip,
                                     cairo_clip_path_t *clip_path)
 {
-    if (clip_path->prev)
-        _cairo_clip_path_reapply_clip_path (clip, clip_path->prev);
+    cairo_status_t status;
 
-    _cairo_clip_intersect_path (clip,
-                                &clip_path->path,
-                                clip_path->fill_rule,
-                                clip_path->tolerance,
-                                clip_path->antialias);
+    if (clip_path->prev) {
+        status = _cairo_clip_path_reapply_clip_path (clip, clip_path->prev);
+	if (status && status != CAIRO_INT_STATUS_UNSUPPORTED)
+	    return status;
+    }
+
+    return _cairo_clip_intersect_path (clip,
+                                       &clip_path->path,
+				       clip_path->fill_rule,
+				       clip_path->tolerance,
+				       clip_path->antialias);
 }
 
-void
+cairo_status_t
 _cairo_clip_init_deep_copy (cairo_clip_t    *clip,
                             cairo_clip_t    *other,
                             cairo_surface_t *target)
 {
+    cairo_status_t status;
+
     _cairo_clip_init (clip, target);
 
     if (other->mode != clip->mode) {
         /* We should reapply the original clip path in this case, and let
          * whatever the right handling is happen */
     } else {
-        if (other->region) {
-            clip->region = pixman_region_create ();
-            pixman_region_copy (clip->region, other->region);
+        if (other->has_region) {
+            status = _cairo_region_copy (&clip->region, &other->region);
+	    if (status)
+		goto BAIL;
+
+	    clip->has_region = TRUE;
         }
 
         if (other->surface) {
-            _cairo_surface_clone_similar (target, other->surface,
-					  other->surface_rect.x,
-					  other->surface_rect.y,
-					  other->surface_rect.width,
-					  other->surface_rect.height,
-					  &clip->surface);
+            status = _cairo_surface_clone_similar (target, other->surface,
+					           other->surface_rect.x,
+						   other->surface_rect.y,
+						   other->surface_rect.width,
+						   other->surface_rect.height,
+						   &clip->surface);
+	    if (status)
+		goto BAIL;
+
             clip->surface_rect = other->surface_rect;
         }
 
         if (other->path) {
-            _cairo_clip_path_reapply_clip_path (clip, other->path);
+            status = _cairo_clip_path_reapply_clip_path (clip, other->path);
+	    if (status && status != CAIRO_INT_STATUS_UNSUPPORTED)
+		goto BAIL;
         }
     }
+
+    return CAIRO_STATUS_SUCCESS;
+
+BAIL:
+    if (clip->has_region)
+	_cairo_region_fini (&clip->region);
+    if (clip->surface)
+	cairo_surface_destroy (clip->surface);
+
+    return status;
 }
 
 const cairo_rectangle_list_t _cairo_rectangles_nil =
@@ -587,19 +683,24 @@ static const cairo_rectangle_list_t _cairo_rectangles_not_representable =
   { CAIRO_STATUS_CLIP_NOT_REPRESENTABLE, NULL, 0 };
 
 static cairo_bool_t
-_cairo_clip_rect_to_user (cairo_gstate_t *gstate,
-                          double x, double y, double width, double height,
-                          cairo_rectangle_t *rectangle)
+_cairo_clip_int_rect_to_user (cairo_gstate_t *gstate,
+			      cairo_rectangle_int_t *clip_rect,
+			      cairo_rectangle_t *user_rect)
 {
-    double x2 = x + width;
-    double y2 = y + height;
     cairo_bool_t is_tight;
 
-    _cairo_gstate_backend_to_user_rectangle (gstate, &x, &y, &x2, &y2, &is_tight);
-    rectangle->x = x;
-    rectangle->y = y;
-    rectangle->width = x2 - x;
-    rectangle->height = y2 - y;
+    double x1 = clip_rect->x;
+    double y1 = clip_rect->y;
+    double x2 = clip_rect->x + clip_rect->width;
+    double y2 = clip_rect->y + clip_rect->height;
+
+    _cairo_gstate_backend_to_user_rectangle (gstate, &x1, &y1, &x2, &y2, &is_tight);
+
+    user_rect->x = x1;
+    user_rect->y = y1;
+    user_rect->width = x2 - x1;
+    user_rect->height = y2 - y1;
+
     return is_tight;
 }
 
@@ -607,50 +708,71 @@ cairo_private cairo_rectangle_list_t*
 _cairo_clip_copy_rectangle_list (cairo_clip_t *clip, cairo_gstate_t *gstate)
 {
     cairo_rectangle_list_t *list;
-    cairo_rectangle_t *rectangles;
-    int n_boxes;
+    cairo_rectangle_t *rectangles = NULL;
+    int n_boxes = 0;
+
+    if (clip->all_clipped)
+	goto DONE;
 
     if (clip->path || clip->surface)
-        return (cairo_rectangle_list_t*) &_cairo_rectangles_not_representable;
+	return (cairo_rectangle_list_t*) &_cairo_rectangles_not_representable;
 
-    n_boxes = clip->region ? pixman_region_num_rects (clip->region) : 1;
-    rectangles = malloc (sizeof (cairo_rectangle_t)*n_boxes);
-    if (rectangles == NULL)
-        return (cairo_rectangle_list_t*) &_cairo_rectangles_nil;
-
-    if (clip->region) {
-        pixman_box16_t *boxes;
+    if (clip->has_region) {
+	cairo_box_int_t *boxes;
         int i;
-        
-        boxes = pixman_region_rects (clip->region);
-        for (i = 0; i < n_boxes; ++i) {
-            if (!_cairo_clip_rect_to_user(gstate, boxes[i].x1, boxes[i].y1,
-                                          boxes[i].x2 - boxes[i].x1,
-                                          boxes[i].y2 - boxes[i].y1,
-                                          &rectangles[i])) {
-                free (rectangles);
-                return (cairo_rectangle_list_t*)
-                    &_cairo_rectangles_not_representable;
-            }
-        }
+
+	if (_cairo_region_get_boxes (&clip->region, &n_boxes, &boxes) != CAIRO_STATUS_SUCCESS)
+	    return (cairo_rectangle_list_t*) &_cairo_rectangles_nil;
+
+	if (n_boxes) {
+	    rectangles = _cairo_malloc_ab (n_boxes, sizeof (cairo_rectangle_t));
+	    if (rectangles == NULL) {
+		_cairo_region_boxes_fini (&clip->region, boxes);
+		_cairo_error_throw (CAIRO_STATUS_NO_MEMORY);
+		return (cairo_rectangle_list_t*) &_cairo_rectangles_nil;
+	    }
+
+	    for (i = 0; i < n_boxes; ++i) {
+		cairo_rectangle_int_t clip_rect = { boxes[i].p1.x, boxes[i].p1.y,
+						    boxes[i].p2.x - boxes[i].p1.x,
+						    boxes[i].p2.y - boxes[i].p1.y };
+
+		if (!_cairo_clip_int_rect_to_user(gstate, &clip_rect, &rectangles[i])) {
+		    _cairo_region_boxes_fini (&clip->region, boxes);
+		    free (rectangles);
+		    return (cairo_rectangle_list_t*) &_cairo_rectangles_not_representable;
+		}
+	    }
+	}
+
+	_cairo_region_boxes_fini (&clip->region, boxes);
     } else {
-        cairo_rectangle_int16_t extents;
-        _cairo_surface_get_extents (_cairo_gstate_get_target (gstate), &extents);
-        if (!_cairo_clip_rect_to_user(gstate, extents.x, extents.y,
-                                      extents.width, extents.height,
-                                      rectangles)) {
-            free (rectangles);
-            return (cairo_rectangle_list_t*)
-                &_cairo_rectangles_not_representable;
-        }
+        cairo_rectangle_int_t extents;
+
+	n_boxes = 1;
+
+	rectangles = malloc(sizeof (cairo_rectangle_t));
+	if (rectangles == NULL) {
+	    _cairo_error_throw (CAIRO_STATUS_NO_MEMORY);
+	    return (cairo_rectangle_list_t*) &_cairo_rectangles_nil;
+	}
+
+	if (_cairo_surface_get_extents (_cairo_gstate_get_target (gstate), &extents) ||
+	    !_cairo_clip_int_rect_to_user(gstate, &extents, rectangles))
+	{
+	    free (rectangles);
+	    return (cairo_rectangle_list_t*) &_cairo_rectangles_not_representable;
+	}
     }
 
+ DONE:
     list = malloc (sizeof (cairo_rectangle_list_t));
     if (list == NULL) {
         free (rectangles);
+	_cairo_error_throw (CAIRO_STATUS_NO_MEMORY);
         return (cairo_rectangle_list_t*) &_cairo_rectangles_nil;
     }
-    
+
     list->status = CAIRO_STATUS_SUCCESS;
     list->rectangles = rectangles;
     list->num_rectangles = n_boxes;
