@@ -35,10 +35,9 @@
  *    Claudio Ciccani <klan@users.sf.net>
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <math.h>
-#include <assert.h>
+#include "cairoint.h"
+
+#include "cairo-directfb.h"
 
 #include <directfb.h>
 
@@ -47,9 +46,6 @@
 #include <direct/memcpy.h>
 #include <direct/util.h>
 
-#include "cairo-directfb.h"
-#include "cairoint.h"
-
 
 /*
  * Rectangle causes problems (see bugs 361377, 359553, 359243 in Gnome BTS).
@@ -72,7 +68,7 @@
 #define DFB_SHOW_GLYPHS 1
 
 
-D_DEBUG_DOMAIN (Cairo_DirectFB, "Cairo/DirectFB", "Cairo DirectFB backend");
+D_DEBUG_DOMAIN (Cairo_DirectFB, "Cairo/DirectFB", "Cairo DirectFB backend")
 
 /*****************************************************************************/
 
@@ -324,9 +320,9 @@ _directfb_buffer_surface_create (IDirectFB             *dfb,
 
 static cairo_status_t
 _directfb_acquire_surface (cairo_directfb_surface_t *surface, 
-                                              cairo_rectangle_int16_t *intrest_rec,
+                                              cairo_rectangle_int_t *intrest_rec,
                                               cairo_image_surface_t **image_out,
-                                              cairo_rectangle_int16_t *image_rect_out,
+                                              cairo_rectangle_int_t *image_rect_out,
                                               void                  **image_extra,
                                               DFBSurfaceLockFlags       lock_flags)
 {   
@@ -337,7 +333,7 @@ _directfb_acquire_surface (cairo_directfb_surface_t *surface,
     cairo_format_t            cairo_format;
     cairo_format = surface->format;    
         
-    if (surface->format == -1) {
+    if (surface->format == (cairo_format_t) -1) {
         if( intrest_rec ) {
             source_rect.x = intrest_rec->x;
             source_rect.y = intrest_rec->y;
@@ -394,7 +390,7 @@ _directfb_acquire_surface (cairo_directfb_surface_t *surface,
       if( buffer != surface->dfbsurface) 
         buffer->Release(buffer);
     }
-    return CAIRO_STATUS_NO_MEMORY;
+    return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 }
 
 
@@ -418,8 +414,10 @@ _cairo_directfb_surface_create_similar (void            *abstract_src,
 
     format = _cairo_format_from_content (content);             
     surface = calloc (1, sizeof(cairo_directfb_surface_t));
-    if (!surface) 
+    if (!surface) {
+	_cairo_error_throw (CAIRO_STATUS_NO_MEMORY);
         return NULL;
+    }
    
     surface->dfbsurface = _directfb_buffer_surface_create (source->dfb,
                                               cairo_to_directfb_format (format),
@@ -503,9 +501,9 @@ _cairo_directfb_surface_release_source_image (void                  *abstract_su
 
 static cairo_status_t
 _cairo_directfb_surface_acquire_dest_image (void                   *abstract_surface,
-                                            cairo_rectangle_int16_t      *interest_rect,
+                                            cairo_rectangle_int_t      *interest_rect,
                                             cairo_image_surface_t **image_out,
-                                            cairo_rectangle_int16_t      *image_rect_out,
+                                            cairo_rectangle_int_t      *image_rect_out,
                                             void                  **image_extra)
 {
     cairo_directfb_surface_t *surface = abstract_surface;
@@ -519,9 +517,9 @@ _cairo_directfb_surface_acquire_dest_image (void                   *abstract_sur
 
 static void
 _cairo_directfb_surface_release_dest_image (void                  *abstract_surface,
-                                            cairo_rectangle_int16_t     *interest_rect,
+                                            cairo_rectangle_int_t     *interest_rect,
                                             cairo_image_surface_t *image,
-                                            cairo_rectangle_int16_t     *image_rect,
+                                            cairo_rectangle_int_t     *image_rect,
                                             void                  *image_extra)
 {
     cairo_directfb_surface_t *surface = abstract_surface;
@@ -532,12 +530,12 @@ _cairo_directfb_surface_release_dest_image (void                  *abstract_surf
     buffer->Unlock (buffer);
 
     if (surface->dfbsurface != buffer) {
-        DFBRegion region = { x1:interest_rect->x, y1:interest_rect->y,
-                x2:interest_rect->x+interest_rect->width-1,
-                y2:interest_rect->y+interest_rect->height-1 };
+        DFBRegion region = { .x1 = interest_rect->x, .y1 = interest_rect->y,
+                .x2 = interest_rect->x+interest_rect->width-1,
+                .y2 = interest_rect->y+interest_rect->height-1 };
         surface->dfbsurface->SetClip (surface->dfbsurface, &region);
-        //surface->dfbsurface->SetBlittingFlags (surface->dfbsurface,
-         //               DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE);
+        /* surface->dfbsurface->SetBlittingFlags (surface->dfbsurface,
+                        DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_COLORIZE); */
         surface->dfbsurface->Blit (surface->dfbsurface,buffer,NULL,
                         image_rect->x,image_rect->y);
         buffer->Release (buffer);
@@ -578,14 +576,14 @@ _cairo_directfb_surface_clone_similar (void             *abstract_surface,
                             _cairo_content_from_format (image_src->format),
                             image_src->width, image_src->height);
         if (!clone)
-            return CAIRO_STATUS_NO_MEMORY;
+            return _cairo_error (CAIRO_STATUS_NO_MEMORY);
             
         ret = clone->dfbsurface->Lock (clone->dfbsurface, 
                                    DSLF_WRITE, (void *)&dst, &pitch);
         if (ret) {
             DirectFBError ("IDirectFBSurface::Lock()", ret);
             cairo_surface_destroy ((cairo_surface_t *)clone);
-            return CAIRO_STATUS_NO_MEMORY;
+            return _cairo_error (CAIRO_STATUS_NO_MEMORY);
         }
 
 	dst += pitch * src_y;
@@ -668,7 +666,7 @@ _directfb_prepare_composite (cairo_directfb_surface_t   *dst,
             if (sblend == DSBF_ONE) {
                 flags |= DSBLIT_BLEND_ALPHACHANNEL;                   
                 sblend = DSBF_SRCALPHA;
-                //dblend = DSBF_INVSRCALPHA;
+                /* dblend = DSBF_INVSRCALPHA; */
             }
         }
             
@@ -689,7 +687,7 @@ _directfb_prepare_composite (cairo_directfb_surface_t   *dst,
             dst->color = _cairo_directfb_surface_create_similar (dst,
                                                 CAIRO_CONTENT_COLOR_ALPHA, 1, 1);
             if (!dst->color)
-                return CAIRO_STATUS_NO_MEMORY;
+                return _cairo_error (CAIRO_STATUS_NO_MEMORY);
         }
         
         src = (cairo_directfb_surface_t *)dst->color;
@@ -760,7 +758,7 @@ _cairo_directfb_surface_composite (cairo_operator_t  op,
                                    unsigned int      height)
 {
     cairo_directfb_surface_t   *dst = abstract_dst;
-    cairo_directfb_surface_t   *src;
+    cairo_directfb_surface_t   *src = NULL; /* hide compiler warning */
     cairo_surface_attributes_t  src_attr;
     cairo_matrix_t             *m;
     cairo_status_t              ret;
@@ -915,7 +913,7 @@ static cairo_int_status_t
 _cairo_directfb_surface_fill_rectangles (void                *abstract_surface,
                                          cairo_operator_t     op,
                                          const cairo_color_t *color,
-                                         cairo_rectangle_int16_t   *rects,
+                                         cairo_rectangle_int_t   *rects,
                                          int                  n_rects)
 {
     cairo_directfb_surface_t *dst = abstract_surface;
@@ -1099,8 +1097,8 @@ _cairo_directfb_surface_composite_trapezoids (cairo_operator_t   op,
 #endif /* DFB_COMPOSITE_TRAPEZOIDS */
 
 static cairo_int_status_t
-_cairo_directfb_surface_set_clip_region (void              *abstract_surface,
-                                         pixman_region16_t *region)
+_cairo_directfb_surface_set_clip_region (void           *abstract_surface,
+                                         cairo_region_t *region)
 {
     cairo_directfb_surface_t *surface = abstract_surface;
     
@@ -1109,29 +1107,36 @@ _cairo_directfb_surface_set_clip_region (void              *abstract_surface,
                 __FUNCTION__, surface, region);
     
     if (region) {
-        pixman_box16_t *boxes   = pixman_region_rects (region);
-        int             n_boxes = pixman_region_num_rects (region);
-        int             i;
-        
+        cairo_box_int_t *boxes;
+        int n_boxes, i;
+	cairo_status_t status;
+
+        status = _cairo_region_get_boxes (region, &n_boxes, &boxes);
+	if (status)
+	    return status;
+
         if (surface->n_clips != n_boxes) {
             if( surface->clips )
                 free (surface->clips);
             
-            surface->clips = malloc (n_boxes * sizeof(DFBRegion));
+            surface->clips = _cairo_malloc_ab (n_boxes, sizeof(DFBRegion));
             if (!surface->clips) {
+                _cairo_region_boxes_fini (region, boxes);
                 surface->n_clips = 0;
-                return CAIRO_STATUS_NO_MEMORY;
+		return _cairo_error (CAIRO_STATUS_NO_MEMORY);
             }
         
             surface->n_clips = n_boxes;
         }
         
         for (i = 0; i < n_boxes; i++) {
-            surface->clips[i].x1 = boxes[i].x1;
-            surface->clips[i].y1 = boxes[i].y1;
-            surface->clips[i].x2 = boxes[i].x2;
-            surface->clips[i].y2 = boxes[i].y2;
+            surface->clips[i].x1 = boxes[i].p1.x;
+            surface->clips[i].y1 = boxes[i].p1.y;
+            surface->clips[i].x2 = boxes[i].p2.x;
+            surface->clips[i].y2 = boxes[i].p2.y;
         }
+
+        _cairo_region_boxes_fini (region, boxes);
     }
     else {
         if (surface->clips) {
@@ -1146,7 +1151,7 @@ _cairo_directfb_surface_set_clip_region (void              *abstract_surface,
 
 static cairo_int_status_t
 _cairo_directfb_abstract_surface_get_extents (void              *abstract_surface,
-                                              cairo_rectangle_int16_t *rectangle)
+                                              cairo_rectangle_int_t *rectangle)
 {
     cairo_directfb_surface_t *surface = abstract_surface;
     
@@ -1180,7 +1185,7 @@ _cairo_directfb_surface_mark_dirty_rectangle (void *abstract_surface,
     if( !surface->dirty_region ) 
             surface->dirty_region = malloc(sizeof(DFBRegion));
     if (!dirty_region)
-            return CAIRO_STATUS_NO_MEMORY;
+            return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 #endif 
     return CAIRO_STATUS_SUCCESS;
 }
@@ -1210,16 +1215,19 @@ _directfb_allocate_font_cache (IDirectFB *dfb, int width, int height)
     cairo_directfb_font_cache_t *cache;
 
     cache = calloc (1, sizeof(cairo_directfb_font_cache_t));
-    if (!cache)
-        return NULL;
+    if (!cache) {
+	_cairo_error_throw (CAIRO_STATUS_NO_MEMORY);
+	return NULL;
+    }
 
     cache->dfbsurface = _directfb_buffer_surface_create( dfb, DSPF_A8, width, height);
     if (!cache->dfbsurface) {
         free (cache);
+	_cairo_error_throw (CAIRO_STATUS_NO_MEMORY);
         return NULL;
     }
 
-    //dfb->AddRef (dfb);
+    /* dfb->AddRef (dfb); */
     cache->dfb = dfb;
 
     cache->width  = width;
@@ -1283,8 +1291,8 @@ _directfb_acquire_font_cache (cairo_directfb_surface_t     *surface,
                 return CAIRO_INT_STATUS_UNSUPPORTED;
         }
         
-        points[n].x = _cairo_lround (glyphs[i].x + img->base.device_transform.x0);
-        points[n].y = _cairo_lround (glyphs[i].y + img->base.device_transform.y0);
+        points[n].x = _cairo_lround (glyphs[i].x - img->base.device_transform.x0);
+        points[n].y = _cairo_lround (glyphs[i].y - img->base.device_transform.y0);
         
         if (points[n].x >= surface->width  ||
             points[n].y >= surface->height ||
@@ -1313,7 +1321,7 @@ _directfb_acquire_font_cache (cairo_directfb_surface_t     *surface,
             /* Remember glyph location */ 
             rect = malloc (sizeof(DFBRectangle));
             if (!rect)
-                return CAIRO_STATUS_NO_MEMORY;
+		return _cairo_error (CAIRO_STATUS_NO_MEMORY);
             *rect = rects[n];
             
             scaled_glyph->surface_private = rect;
@@ -1351,7 +1359,7 @@ _directfb_acquire_font_cache (cairo_directfb_surface_t     *surface,
             
             new_cache = _directfb_allocate_font_cache (surface->dfb, w, h);
             if (!new_cache)
-                return CAIRO_STATUS_NO_MEMORY;
+		return _cairo_error (CAIRO_STATUS_NO_MEMORY);
             
             new_cache->dfbsurface->Blit (new_cache->dfbsurface,
                                      cache->dfbsurface, NULL, 0, 0);
@@ -1365,8 +1373,8 @@ _directfb_acquire_font_cache (cairo_directfb_surface_t     *surface,
                     "Allocating font cache (%dx%d).\n", w, h);
         
         cache = _directfb_allocate_font_cache (surface->dfb, w, h);
-        if (!cache)
-            return CAIRO_STATUS_NO_MEMORY;
+	if (!cache)
+	    return _cairo_error (CAIRO_STATUS_NO_MEMORY);
             
         scaled_font->surface_backend = &cairo_directfb_surface_backend;
         scaled_font->surface_private = cache;
@@ -1378,7 +1386,7 @@ _directfb_acquire_font_cache (cairo_directfb_surface_t     *surface,
     
         if (cache->dfbsurface->Lock (cache->dfbsurface, 
                                  DSLF_WRITE, (void *)&data, &pitch))
-            return CAIRO_STATUS_NO_MEMORY;
+            return _cairo_error (CAIRO_STATUS_NO_MEMORY);
     
         for (i = 0; i < num_chars; i++) {
             cairo_image_surface_t *img  = chars[i]->surface;
@@ -1515,6 +1523,17 @@ _cairo_directfb_surface_show_glyphs ( void                 *abstract_dst,
 #endif /* DFB_SHOW_GLYPHS */
 
 
+static cairo_bool_t
+_cairo_directfb_surface_is_similar (void *surface_a,
+	                           void *surface_b,
+				   cairo_content_t content)
+{
+    cairo_directfb_surface_t *a = (cairo_directfb_surface_t *) surface_a;
+    cairo_directfb_surface_t *b = (cairo_directfb_surface_t *) surface_b;
+
+    return a->dfb == b->dfb;
+}
+
 static cairo_surface_backend_t cairo_directfb_surface_backend = {
          CAIRO_SURFACE_TYPE_DIRECTFB, /*type*/
         _cairo_directfb_surface_create_similar,/*create_similar*/
@@ -1564,7 +1583,9 @@ static cairo_surface_backend_t cairo_directfb_surface_backend = {
 #else
         NULL, /* show_glyphs */
 #endif
-        NULL /* snapshot */
+        NULL, /* snapshot */
+	_cairo_directfb_surface_is_similar,
+	NULL /* reset */
 };
 
 
@@ -1615,8 +1636,10 @@ cairo_directfb_surface_create (IDirectFB *dfb, IDirectFBSurface *dfbsurface)
     cairo_directfb_surface_backend_init (dfb);
         
     surface = calloc (1, sizeof(cairo_directfb_surface_t));
-    if (!surface)
-        return NULL;
+    if (!surface) {
+	_cairo_error_throw (CAIRO_STATUS_NO_MEMORY);
+	return NULL;
+    }
         
     dfbsurface->GetPixelFormat (dfbsurface, &format);
     _cairo_surface_init (&surface->base, &cairo_directfb_surface_backend,
