@@ -34,7 +34,7 @@ typedef struct _cairo_glitz_surface {
     glitz_surface_t   *surface;
     glitz_format_t    *format;
     cairo_bool_t      has_clip;
-    pixman_region16_t clip;
+    cairo_region_t    clip;
 } cairo_glitz_surface_t;
 
 static const cairo_surface_backend_t *
@@ -47,7 +47,7 @@ _cairo_glitz_surface_finish (void *abstract_surface)
 
     if (surface->has_clip) {
         glitz_surface_set_clip_region (surface->surface, 0, 0, NULL, 0);
-        pixman_region_fini (&surface->clip);
+        _cairo_region_fini (&surface->clip);
     }
 
     glitz_surface_destroy (surface->surface);
@@ -89,7 +89,7 @@ _cairo_glitz_surface_create_similar (void	    *abstract_src,
 	glitz_find_standard_format (drawable,
 				    _glitz_format_from_content (content));
     if (!gformat) {
-	_cairo_error (CAIRO_STATUS_NO_MEMORY);
+	_cairo_error_throw (CAIRO_STATUS_NO_MEMORY);
 	return (cairo_surface_t*) &_cairo_surface_nil;
     }
 
@@ -99,7 +99,7 @@ _cairo_glitz_surface_create_similar (void	    *abstract_src,
 				    0, NULL);
 
     if (surface == NULL) {
-	_cairo_error (CAIRO_STATUS_NO_MEMORY);
+	_cairo_error_throw (CAIRO_STATUS_NO_MEMORY);
 	return (cairo_surface_t*) &_cairo_surface_nil;
     }
 
@@ -110,19 +110,92 @@ _cairo_glitz_surface_create_similar (void	    *abstract_src,
     return crsurface;
 }
 
+static cairo_bool_t
+_CAIRO_MASK_FORMAT (cairo_format_masks_t *masks, cairo_format_t *format)
+{
+    switch (masks->bpp) {
+    case 32:
+	if (masks->alpha_mask == 0xff000000 &&
+	    masks->red_mask == 0x00ff0000 &&
+	    masks->green_mask == 0x0000ff00 &&
+	    masks->blue_mask == 0x000000ff)
+	{
+	    *format = CAIRO_FORMAT_ARGB32;
+	    return TRUE;
+	}
+	if (masks->alpha_mask == 0x00000000 &&
+	    masks->red_mask == 0x00ff0000 &&
+	    masks->green_mask == 0x0000ff00 &&
+	    masks->blue_mask == 0x000000ff)
+	{
+	    *format = CAIRO_FORMAT_RGB24;
+	    return TRUE;
+	}
+	break;
+    case 8:
+	if (masks->alpha_mask == 0xff)
+	{
+	    *format = CAIRO_FORMAT_A8;
+	    return TRUE;
+	}
+	break;
+    case 1:
+	if (masks->alpha_mask == 0x1)
+	{
+	    *format = CAIRO_FORMAT_A1;
+	    return TRUE;
+	}
+	break;
+    }
+    return FALSE;
+}
+
+static glitz_box_t *
+_cairo_glitz_get_boxes_from_region (cairo_region_t *region, int *nboxes)
+{
+    cairo_box_int_t *cboxes;
+    glitz_box_t *gboxes;
+    int n, i;
+
+    if (_cairo_region_get_boxes (region, &n, &cboxes) != CAIRO_STATUS_SUCCESS)
+        return NULL;
+
+    *nboxes = n;
+    if (n == 0)
+        return NULL;
+
+    gboxes = _cairo_malloc_ab (n, sizeof(glitz_box_t));
+    if (gboxes == NULL) {
+	_cairo_error_throw (CAIRO_STATUS_NO_MEMORY);
+        goto done;
+    }
+
+    for (i = 0; i < n; i++) {
+        gboxes[i].x1 = cboxes[i].p1.x;
+        gboxes[i].y1 = cboxes[i].p1.y;
+        gboxes[i].x2 = cboxes[i].p2.x;
+        gboxes[i].y2 = cboxes[i].p2.y;
+    }
+
+done:
+    _cairo_region_boxes_fini (region, cboxes);
+    return gboxes;
+}
+
 static cairo_status_t
 _cairo_glitz_surface_get_image (cairo_glitz_surface_t   *surface,
-				cairo_rectangle_int16_t *interest,
+				cairo_rectangle_int_t   *interest,
 				cairo_image_surface_t  **image_out,
-				cairo_rectangle_int16_t *rect_out)
+				cairo_rectangle_int_t   *rect_out)
 {
     cairo_image_surface_t *image;
     int			  x1, y1, x2, y2;
     int			  width, height;
     unsigned char	  *pixels;
-    cairo_format_masks_t  format;
+    cairo_format_masks_t  masks;
     glitz_buffer_t	  *buffer;
     glitz_pixel_format_t  pf;
+    cairo_format_t	  format;
 
     x1 = 0;
     y1 = 0;
@@ -160,51 +233,51 @@ _cairo_glitz_surface_get_image (cairo_glitz_surface_t   *surface,
 
     if (surface->format->color.fourcc == GLITZ_FOURCC_RGB) {
 	if (surface->format->color.red_size > 0) {
-	    format.bpp = 32;
+	    masks.bpp = 32;
 
 	    if (surface->format->color.alpha_size > 0)
-		format.alpha_mask = 0xff000000;
+		masks.alpha_mask = 0xff000000;
 	    else
-		format.alpha_mask = 0x0;
+		masks.alpha_mask = 0x0;
 
-	    format.red_mask = 0xff0000;
-	    format.green_mask = 0xff00;
-	    format.blue_mask = 0xff;
+	    masks.red_mask = 0xff0000;
+	    masks.green_mask = 0xff00;
+	    masks.blue_mask = 0xff;
 	} else {
-	    format.bpp = 8;
-	    format.blue_mask = format.green_mask = format.red_mask = 0x0;
-	    format.alpha_mask = 0xff;
+	    masks.bpp = 8;
+	    masks.blue_mask = masks.green_mask = masks.red_mask = 0x0;
+	    masks.alpha_mask = 0xff;
 	}
     } else {
-	format.bpp = 32;
-	format.alpha_mask = 0xff000000;
-	format.red_mask = 0xff0000;
-	format.green_mask = 0xff00;
-	format.blue_mask = 0xff;
+	masks.bpp = 32;
+	masks.alpha_mask = 0xff000000;
+	masks.red_mask = 0xff0000;
+	masks.green_mask = 0xff00;
+	masks.blue_mask = 0xff;
     }
 
     pf.fourcc = GLITZ_FOURCC_RGB;
-    pf.masks.bpp = format.bpp;
-    pf.masks.alpha_mask = format.alpha_mask;
-    pf.masks.red_mask = format.red_mask;
-    pf.masks.green_mask = format.green_mask;
-    pf.masks.blue_mask = format.blue_mask;
+    pf.masks.bpp = masks.bpp;
+    pf.masks.alpha_mask = masks.alpha_mask;
+    pf.masks.red_mask = masks.red_mask;
+    pf.masks.green_mask = masks.green_mask;
+    pf.masks.blue_mask = masks.blue_mask;
     pf.xoffset = 0;
     pf.skip_lines = 0;
 
     /* XXX: we should eventually return images with negative stride,
        need to verify that libpixman have no problem with this first. */
-    pf.bytes_per_line = (((width * format.bpp) / 8) + 3) & -4;
+    pf.bytes_per_line = (((width * masks.bpp) / 8) + 3) & -4;
     pf.scanline_order = GLITZ_PIXEL_SCANLINE_ORDER_TOP_DOWN;
 
-    pixels = malloc (height * pf.bytes_per_line);
+    pixels = _cairo_malloc_ab (height, pf.bytes_per_line);
     if (!pixels)
-	return CAIRO_STATUS_NO_MEMORY;
+	return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 
     buffer = glitz_buffer_create_for_data (pixels);
     if (!buffer) {
 	free (pixels);
-	return CAIRO_STATUS_NO_MEMORY;
+	return _cairo_error (CAIRO_STATUS_NO_MEMORY);
     }
 
     /* clear out the glitz clip; the clip affects glitz_get_pixels */
@@ -223,22 +296,47 @@ _cairo_glitz_surface_get_image (cairo_glitz_surface_t   *surface,
     /* restore the clip, if any */
     if (surface->has_clip) {
 	glitz_box_t *box;
-	int	    n;
+        int n;
+
+        box = _cairo_glitz_get_boxes_from_region (&surface->clip, &n);
+        if (box == NULL && n != 0) {
+            free (pixels);
+            return _cairo_error (CAIRO_STATUS_NO_MEMORY);
+        }
 
-	box = (glitz_box_t *) pixman_region_rects (&surface->clip);
-	n = pixman_region_num_rects (&surface->clip);
 	glitz_surface_set_clip_region (surface->surface, 0, 0, box, n);
+
+        free (box);
     }
 
-    image = (cairo_image_surface_t *)
-	_cairo_image_surface_create_with_masks (pixels,
-						&format,
-						width, height,
-						pf.bytes_per_line);
-    if (image->base.status)
-    {
-	free (pixels);
-	return CAIRO_STATUS_NO_MEMORY;
+    /*
+     * Prefer to use a standard pixman format instead of the
+     * general masks case.
+     */
+    if (_CAIRO_MASK_FORMAT (&masks, &format)) {
+	image = (cairo_image_surface_t *)
+	    cairo_image_surface_create_for_data (pixels,
+						 format,
+						 x2 - x1,
+						 y2 - y1,
+						 pf.bytes_per_line);
+	if (image->base.status)
+	    goto FAIL;
+    } else {
+	/*
+	 * XXX This can't work.  We must convert the data to one of the
+	 * supported pixman formats.  Pixman needs another function
+	 * which takes data in an arbitrary format and converts it
+	 * to something supported by that library.
+	 */
+	image = (cairo_image_surface_t *)
+	    _cairo_image_surface_create_with_masks (pixels,
+						    &masks,
+						    x2 - x1,
+						    y2 - y1,
+						    pf.bytes_per_line);
+	if (image->base.status)
+	    goto FAIL;
     }
 
     _cairo_image_surface_assume_ownership_of_data (image);
@@ -246,6 +344,47 @@ _cairo_glitz_surface_get_image (cairo_glitz_surface_t   *surface,
     *image_out = image;
 
     return CAIRO_STATUS_SUCCESS;
+
+FAIL:
+    free (pixels);
+    return _cairo_error (CAIRO_STATUS_NO_MEMORY);
+}
+
+static void
+cairo_format_get_masks (cairo_format_t  format,
+			uint32_t       *bpp,
+                        uint32_t       *alpha,
+			uint32_t       *red,
+			uint32_t       *green,
+			uint32_t       *blue)
+{
+    *red = 0x0;
+    *green = 0x0;
+    *blue = 0x0;
+    *alpha = 0x0;
+
+    switch (format)
+    {
+    case CAIRO_FORMAT_ARGB32:
+        *alpha = 0xff000000;
+    case CAIRO_FORMAT_RGB24:
+    default:
+	*bpp =   32;
+	*red =   0x00ff0000;
+	*green = 0x0000ff00;
+	*blue =  0x000000ff;
+	break;
+
+    case CAIRO_FORMAT_A8:
+	*bpp = 8;
+        *alpha = 0xff;
+	break;
+
+    case CAIRO_FORMAT_A1:
+	*bpp = 1;
+        *alpha = 0x1;
+	break;
+    }
 }
 
 static cairo_status_t
@@ -261,15 +400,10 @@ _cairo_glitz_surface_set_image (void		      *abstract_surface,
     cairo_glitz_surface_t *surface = abstract_surface;
     glitz_buffer_t	  *buffer;
     glitz_pixel_format_t  pf;
-    pixman_format_t	  *format;
-    unsigned int	  bpp, am, rm, gm, bm;
+    uint32_t		  bpp, am, rm, gm, bm;
     char		  *data;
 
-    format = pixman_image_get_format (image->pixman_image);
-    if (!format)
-	return CAIRO_STATUS_NO_MEMORY;
-
-    pixman_format_get_masks (format, &bpp, &am, &rm, &gm, &bm);
+    cairo_format_get_masks (image->format, &bpp, &am, &rm, &gm, &bm);
 
     pf.fourcc = GLITZ_FOURCC_RGB;
     pf.masks.bpp = bpp;
@@ -296,7 +430,7 @@ _cairo_glitz_surface_set_image (void		      *abstract_surface,
 
     buffer = glitz_buffer_create_for_data (data);
     if (!buffer)
-	return CAIRO_STATUS_NO_MEMORY;
+	return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 
     glitz_set_pixels (surface->surface,
 		      x_dst, y_dst,
@@ -331,9 +465,9 @@ _cairo_glitz_surface_release_source_image (void              *abstract_surface,
 
 static cairo_status_t
 _cairo_glitz_surface_acquire_dest_image (void                    *abstract_surface,
-					 cairo_rectangle_int16_t *interest_rect,
+					 cairo_rectangle_int_t   *interest_rect,
 					 cairo_image_surface_t  **image_out,
-					 cairo_rectangle_int16_t *image_rect_out,
+					 cairo_rectangle_int_t   *image_rect_out,
 					 void                   **image_extra)
 {
     cairo_glitz_surface_t *surface = abstract_surface;
@@ -353,16 +487,19 @@ _cairo_glitz_surface_acquire_dest_image (void                    *abstract_surfa
 
 static void
 _cairo_glitz_surface_release_dest_image (void                    *abstract_surface,
-					 cairo_rectangle_int16_t *interest_rect,
+					 cairo_rectangle_int_t   *interest_rect,
 					 cairo_image_surface_t   *image,
-					 cairo_rectangle_int16_t *image_rect,
+					 cairo_rectangle_int_t   *image_rect,
 					 void                    *image_extra)
 {
     cairo_glitz_surface_t *surface = abstract_surface;
+    cairo_status_t status;
 
-    _cairo_glitz_surface_set_image (surface, image, 0, 0,
-				    image->width, image->height,
-				    image_rect->x, image_rect->y);
+    status = _cairo_glitz_surface_set_image (surface, image, 0, 0,
+				             image->width, image->height,
+				             image_rect->x, image_rect->y);
+    if (status)
+	status = _cairo_surface_set_error (&surface->base, status);
 
     cairo_surface_destroy (&image->base);
 }
@@ -378,6 +515,7 @@ _cairo_glitz_surface_clone_similar (void	    *abstract_surface,
 {
     cairo_glitz_surface_t *surface = abstract_surface;
     cairo_glitz_surface_t *clone;
+    cairo_status_t status;
 
     if (surface->base.status)
 	return surface->base.status;
@@ -392,8 +530,8 @@ _cairo_glitz_surface_clone_similar (void	    *abstract_surface,
     {
 	cairo_image_surface_t *image_src = (cairo_image_surface_t *) src;
 	cairo_content_t	      content;
-	cairo_rectangle_int16_t image_extent;
-	cairo_rectangle_int16_t extent;
+	cairo_rectangle_int_t image_extent;
+	cairo_rectangle_int_t extent;
 
 	content = _cairo_content_from_format (image_src->format);
 
@@ -402,7 +540,7 @@ _cairo_glitz_surface_clone_similar (void	    *abstract_surface,
 						 image_src->width,
 						 image_src->height);
 	if (clone->base.status)
-	    return CAIRO_STATUS_NO_MEMORY;
+	    return clone->base.status;
 
 	image_extent.x = 0;
 	image_extent.y = 0;
@@ -415,10 +553,14 @@ _cairo_glitz_surface_clone_similar (void	    *abstract_surface,
 
 	_cairo_rectangle_intersect(&extent, &image_extent);
 
-	_cairo_glitz_surface_set_image (clone, image_src,
-					extent.x, extent.y,
-					extent.width, extent.height,
-					extent.x, extent.y);
+	status = _cairo_glitz_surface_set_image (clone, image_src,
+					         extent.x, extent.y,
+						 extent.width, extent.height,
+						 extent.x, extent.y);
+	if (status) {
+	    cairo_surface_destroy (&clone->base);
+	    return status;
+	}
 
 	*clone_out = &clone->base;
 
@@ -590,7 +732,7 @@ _cairo_glitz_pattern_acquire_surface (cairo_pattern_t	              *pattern,
 	    (cairo_gradient_pattern_t *) pattern;
 	char			    *data;
 	glitz_fixed16_16_t	    *params;
-	int			    n_params;
+	unsigned int		    n_params;
 	unsigned int		    *pixels;
 	unsigned int		    i, n_base_params;
 	glitz_buffer_t		    *buffer;
@@ -627,20 +769,33 @@ _cairo_glitz_pattern_acquire_surface (cairo_pattern_t	              *pattern,
 
 	n_params = gradient->n_stops * 3 + n_base_params;
 
-	data = malloc (sizeof (glitz_fixed16_16_t) * n_params +
-		       sizeof (unsigned int) * gradient->n_stops);
+        /* check for int overflow */
+        {
+            int size1, size2;
+            if (n_params >= INT32_MAX / sizeof (glitz_fixed16_16_t) ||
+                gradient->n_stops >= INT32_MAX / sizeof (unsigned int))
+                return _cairo_error (CAIRO_STATUS_NO_MEMORY);
+
+            size1 = n_params * sizeof (glitz_fixed16_16_t);
+            size2 = gradient->n_stops * sizeof (unsigned int);
+
+            if (size1 >= INT32_MAX - size2)
+                return _cairo_error (CAIRO_STATUS_NO_MEMORY);
+
+            data = malloc (size1 + size2);
+        }
+
 	if (!data)
-	    return CAIRO_STATUS_NO_MEMORY;
+	    return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 
 	params = (glitz_fixed16_16_t *) data;
 	pixels = (unsigned int *)
 	    (data + sizeof (glitz_fixed16_16_t) * n_params);
 
 	buffer = glitz_buffer_create_for_data (pixels);
-	if (!buffer)
-	{
+	if (!buffer) {
 	    free (data);
-	    return CAIRO_STATUS_NO_MEMORY;
+	    return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 	}
 
 	src = (cairo_glitz_surface_t *)
@@ -651,16 +806,16 @@ _cairo_glitz_pattern_acquire_surface (cairo_pattern_t	              *pattern,
 	{
 	    glitz_buffer_destroy (buffer);
 	    free (data);
-	    return CAIRO_STATUS_NO_MEMORY;
+	    return src->base.status;
 	}
 
 	for (i = 0; i < gradient->n_stops; i++)
 	{
 	    pixels[i] =
-		(((int) (gradient->stops[i].color.alpha >> 8)) << 24) |
-		(((int) (gradient->stops[i].color.red   >> 8)) << 16) |
-		(((int) (gradient->stops[i].color.green >> 8)) << 8)  |
-		(((int) (gradient->stops[i].color.blue  >> 8)));
+		(((int) (gradient->stops[i].color.alpha_short >> 8)) << 24) |
+		(((int) (gradient->stops[i].color.red_short   >> 8)) << 16) |
+		(((int) (gradient->stops[i].color.green_short >> 8)) << 8)  |
+		(((int) (gradient->stops[i].color.blue_short  >> 8)));
 
 	    params[n_base_params + 3 * i + 0] = gradient->stops[i].x;
 	    params[n_base_params + 3 * i + 1] = i << 16;
@@ -676,22 +831,22 @@ _cairo_glitz_pattern_acquire_surface (cairo_pattern_t	              *pattern,
 	{
 	    cairo_linear_pattern_t *grad = (cairo_linear_pattern_t *) pattern;
 
-	    params[0] = grad->gradient.p1.x;
-	    params[1] = grad->gradient.p1.y;
-	    params[2] = grad->gradient.p2.x;
-	    params[3] = grad->gradient.p2.y;
+	    params[0] = grad->p1.x;
+	    params[1] = grad->p1.y;
+	    params[2] = grad->p2.x;
+	    params[3] = grad->p2.y;
 	    attr->filter = GLITZ_FILTER_LINEAR_GRADIENT;
 	}
 	else
 	{
 	    cairo_radial_pattern_t *grad = (cairo_radial_pattern_t *) pattern;
 
-	    params[0] = grad->gradient.c1.x;
-	    params[1] = grad->gradient.c1.y;
-	    params[2] = grad->gradient.c1.radius;
-	    params[3] = grad->gradient.c2.x;
-	    params[4] = grad->gradient.c2.y;
-	    params[5] = grad->gradient.c2.radius;
+	    params[0] = grad->c1.x;
+	    params[1] = grad->c1.y;
+	    params[2] = grad->r1;
+	    params[3] = grad->c2.x;
+	    params[4] = grad->c2.y;
+	    params[5] = grad->r2;
 	    attr->filter = GLITZ_FILTER_RADIAL_GRADIENT;
 	}
 
@@ -964,7 +1119,7 @@ static cairo_int_status_t
 _cairo_glitz_surface_fill_rectangles (void		      *abstract_dst,
 				      cairo_operator_t	       op,
 				      const cairo_color_t     *color,
-				      cairo_rectangle_int16_t *rects,
+				      cairo_rectangle_int_t   *rects,
 				      int		       n_rects)
 {
     cairo_glitz_surface_t *dst = abstract_dst;
@@ -1012,7 +1167,7 @@ _cairo_glitz_surface_fill_rectangles (void		      *abstract_dst,
 						 (cairo_color_t *) color,
 						 NULL);
 	if (src->base.status)
-	    return CAIRO_STATUS_NO_MEMORY;
+	    return src->base.status;
 
 	glitz_surface_set_fill (src->surface, GLITZ_FILL_REPEAT);
 
@@ -1131,7 +1286,7 @@ _cairo_glitz_surface_composite_trapezoids (cairo_operator_t  op,
 	    if (src_pattern == &tmp_src_pattern.base)
 		_cairo_pattern_fini (&tmp_src_pattern.base);
 
-	    return CAIRO_STATUS_NO_MEMORY;
+	    return mask->base.status;
 	}
 
 	color.red = color.green = color.blue = color.alpha = 0xffff;
@@ -1158,7 +1313,7 @@ _cairo_glitz_surface_composite_trapezoids (cairo_operator_t  op,
 							  &attributes);
 		    if (src_pattern == &tmp_src_pattern.base)
 			_cairo_pattern_fini (&tmp_src_pattern.base);
-		    return CAIRO_STATUS_NO_MEMORY;
+		    return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 		}
 
 		if (buffer)
@@ -1171,7 +1326,7 @@ _cairo_glitz_surface_composite_trapezoids (cairo_operator_t  op,
 							  &attributes);
 		    if (src_pattern == &tmp_src_pattern.base)
 			_cairo_pattern_fini (&tmp_src_pattern.base);
-		    return CAIRO_STATUS_NO_MEMORY;
+		    return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 		}
 	    }
 
@@ -1201,13 +1356,13 @@ _cairo_glitz_surface_composite_trapezoids (cairo_operator_t  op,
 	int		      stride;
 
 	stride = (width + 3) & -4;
-	data = calloc (stride * height, 1);
+	data = calloc (stride, height);
 	if (!data)
 	{
 	    _cairo_glitz_pattern_release_surface (src_pattern, src, &attributes);
 	    if (src_pattern == &tmp_src_pattern.base)
 		_cairo_pattern_fini (&tmp_src_pattern.base);
-	    return CAIRO_STATUS_NO_MEMORY;
+	    return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 	}
 
 	/* using negative stride */
@@ -1222,11 +1377,11 @@ _cairo_glitz_surface_composite_trapezoids (cairo_operator_t  op,
 	{
 	    cairo_surface_destroy (&src->base);
 	    free (data);
-	    return CAIRO_STATUS_NO_MEMORY;
+	    return image->base.status;
 	}
 
 	pixman_add_trapezoids (image->pixman_image, -dst_x, -dst_y,
-			       (pixman_trapezoid_t *) traps, n_traps);
+                               n_traps, (pixman_trapezoid_t *) traps);
 
 	mask = (cairo_glitz_surface_t *)
 	    _cairo_surface_create_similar_scratch (&dst->base,
@@ -1237,12 +1392,16 @@ _cairo_glitz_surface_composite_trapezoids (cairo_operator_t  op,
 	    _cairo_glitz_pattern_release_surface (src_pattern, src, &attributes);
 	    free (data);
 	    cairo_surface_destroy (&image->base);
-	    return CAIRO_STATUS_NO_MEMORY;
+	    return mask->base.status;
 	}
 
-	_cairo_glitz_surface_set_image (mask, image, 0, 0, width, height, 0, 0);
+	status = _cairo_glitz_surface_set_image (mask, image,
+		                                 0, 0, width, height, 0, 0);
 
 	cairo_surface_destroy(&image->base);
+
+	if (status)
+	    return status;
     }
 
     _cairo_glitz_surface_set_attributes (src, &attributes);
@@ -1284,7 +1443,7 @@ _cairo_glitz_surface_composite_trapezoids (cairo_operator_t  op,
 
 static cairo_int_status_t
 _cairo_glitz_surface_set_clip_region (void		*abstract_surface,
-				      pixman_region16_t *region)
+                                      cairo_region_t	*region)
 {
     cairo_glitz_surface_t *surface = abstract_surface;
 
@@ -1292,31 +1451,37 @@ _cairo_glitz_surface_set_clip_region (void		*abstract_surface,
     {
 	glitz_box_t *box;
 	int	    n;
+	cairo_status_t status;
 
 	if (!surface->has_clip) {
-            pixman_region_init (&surface->clip);
+            _cairo_region_init (&surface->clip);
             surface->has_clip = TRUE;
         }
 
-	if (pixman_region_copy (&surface->clip, region) !=
-            PIXMAN_REGION_STATUS_SUCCESS)
-        {
-	    pixman_region_fini (&surface->clip);
+	status = _cairo_region_copy (&surface->clip, region);
+	if (status) {
+            _cairo_region_fini (&surface->clip);
 	    surface->has_clip = FALSE;
-            return CAIRO_STATUS_NO_MEMORY;
+            return status;
         }
 
-	box = (glitz_box_t *) pixman_region_rects (&surface->clip);
-	n = pixman_region_num_rects (&surface->clip);
+        box = _cairo_glitz_get_boxes_from_region (&surface->clip, &n);
+        if (box == NULL && n != 0) {
+            _cairo_region_fini (&surface->clip);
+	    surface->has_clip = FALSE;
+            return _cairo_error (CAIRO_STATUS_NO_MEMORY);
+        }
 
 	glitz_surface_set_clip_region (surface->surface, 0, 0, box, n);
+
+        free (box);
     }
     else
     {
 	glitz_surface_set_clip_region (surface->surface, 0, 0, NULL, 0);
 
 	if (surface->has_clip) {
-	    pixman_region_fini (&surface->clip);
+	    _cairo_region_fini (&surface->clip);
 	    surface->has_clip = FALSE;
         }
     }
@@ -1326,7 +1491,7 @@ _cairo_glitz_surface_set_clip_region (void		*abstract_surface,
 
 static cairo_int_status_t
 _cairo_glitz_surface_get_extents (void		          *abstract_surface,
-				  cairo_rectangle_int16_t *rectangle)
+				  cairo_rectangle_int_t   *rectangle)
 {
     cairo_glitz_surface_t *surface = abstract_surface;
 
@@ -1414,8 +1579,10 @@ _cairo_glitz_area_create (cairo_glitz_root_area_t *root,
     int		       n = 4;
 
     area = malloc (sizeof (cairo_glitz_area_t));
-    if (!area)
+    if (!area) {
+	_cairo_error_throw (CAIRO_STATUS_NO_MEMORY);
 	return NULL;
+    }
 
     area->level   = level;
     area->x	  = x;
@@ -1637,7 +1804,7 @@ _cairo_glitz_root_area_init (cairo_glitz_root_area_t	    *root,
 
     root->area = _cairo_glitz_area_create (root, 0, 0, 0, width, height);
     if (!root->area)
-	return CAIRO_STATUS_NO_MEMORY;
+	return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 
     return CAIRO_STATUS_SUCCESS;
 }
@@ -1750,7 +1917,7 @@ _cairo_glitz_surface_font_init (cairo_glitz_surface_t *surface,
 
     font_private = malloc (sizeof (cairo_glitz_surface_font_private_t));
     if (!font_private)
-	return CAIRO_STATUS_NO_MEMORY;
+	return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 
     font_private->surface = glitz_surface_create (drawable, surface_format,
 						  GLYPH_CACHE_TEXTURE_SIZE,
@@ -1828,7 +1995,6 @@ _cairo_glitz_surface_add_glyph (cairo_glitz_surface_t *surface,
     glitz_point_fixed_t			p1, p2;
     glitz_pixel_format_t		pf;
     glitz_buffer_t			*buffer;
-    pixman_format_t			*format;
     unsigned int			bpp, am, rm, gm, bm;
     cairo_int_status_t			status;
 
@@ -1837,7 +2003,7 @@ _cairo_glitz_surface_add_glyph (cairo_glitz_surface_t *surface,
     {
 	glyph_private = malloc (sizeof (cairo_glitz_surface_glyph_private_t));
 	if (!glyph_private)
-	    return CAIRO_STATUS_NO_MEMORY;
+	    return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 
 	glyph_private->area   = NULL;
 	glyph_private->locked = FALSE;
@@ -1865,10 +2031,6 @@ _cairo_glitz_surface_add_glyph (cairo_glitz_surface_t *surface,
 	return CAIRO_STATUS_SUCCESS;
     }
 
-    format = pixman_image_get_format (glyph_surface->pixman_image);
-    if (!format)
-	return CAIRO_STATUS_NO_MEMORY;
-
     if (_cairo_glitz_area_find (font_private->root.area,
 				glyph_surface->width,
 				glyph_surface->height,
@@ -1885,10 +2047,10 @@ _cairo_glitz_surface_add_glyph (cairo_glitz_surface_t *surface,
     if (!buffer)
     {
 	_cairo_glitz_area_move_out (glyph_private->area);
-	return CAIRO_STATUS_NO_MEMORY;
+	return _cairo_error (CAIRO_STATUS_NO_MEMORY);
     }
 
-    pixman_format_get_masks (format, &bpp, &am, &rm, &gm, &bm);
+    cairo_format_get_masks (glyph_surface->format, &bpp, &am, &rm, &gm, &bm);
 
     pf.fourcc		= GLITZ_FOURCC_RGB;
     pf.masks.bpp        = bpp;
@@ -1992,11 +2154,23 @@ _cairo_glitz_surface_old_show_glyphs (cairo_scaled_font_t *scaled_font,
     if (num_glyphs > N_STACK_BUF)
     {
 	char *data;
+        size_t size1, size2;
 
-	data = malloc (num_glyphs * sizeof (void *) +
-		       num_glyphs * sizeof (glitz_float_t) * 16);
-	if (!data)
+        if ((size_t)num_glyphs >= INT32_MAX / sizeof(void*) ||
+            (size_t)num_glyphs >= INT32_MAX / sizeof(glitz_float_t) ||
+            ((size_t)num_glyphs * sizeof(glitz_float_t)) >= INT32_MAX / 16)
+            goto FAIL1;
+
+        size1 = num_glyphs * sizeof(void *);
+        size2 = num_glyphs * sizeof(glitz_float_t) * 16;
+        if (size1 >= INT32_MAX - size2)
+            goto FAIL1;
+
+	data = malloc (size1 + size2);
+	if (!data) {
+	    _cairo_error_throw (CAIRO_STATUS_NO_MEMORY);
 	    goto FAIL1;
+	}
 
 	scaled_glyphs = (cairo_scaled_glyph_t **) data;
 	vertices = (glitz_float_t *) (data + num_glyphs * sizeof (void *));
@@ -2046,8 +2220,8 @@ _cairo_glitz_surface_old_show_glyphs (cairo_scaled_font_t *scaled_font,
 		x_offset = scaled_glyphs[i]->surface->base.device_transform.x0;
 		y_offset = scaled_glyphs[i]->surface->base.device_transform.y0;
 
-		x1 = _cairo_lround (glyphs[i].x) + x_offset;
-		y1 = _cairo_lround (glyphs[i].y) + y_offset;
+		x1 = _cairo_lround (glyphs[i].x - x_offset);
+		y1 = _cairo_lround (glyphs[i].y - y_offset);
 		x2 = x1 + glyph_private->area->width;
 		y2 = y1 + glyph_private->area->height;
 
@@ -2090,8 +2264,8 @@ _cairo_glitz_surface_old_show_glyphs (cairo_scaled_font_t *scaled_font,
 
 		x_offset = scaled_glyphs[i]->surface->base.device_transform.x0;
 		y_offset = scaled_glyphs[i]->surface->base.device_transform.y0;
-		x1 = _cairo_lround (glyphs[i].x) + x_offset;
-		y1 = _cairo_lround (glyphs[i].y) + y_offset;
+		x1 = _cairo_lround (glyphs[i].x - x_offset);
+		y1 = _cairo_lround (glyphs[i].y - y_offset);
 
 		glitz_composite (_glitz_operator (op),
 				 src->surface,
@@ -2284,7 +2458,7 @@ cairo_glitz_surface_create (glitz_surface_t *surface)
 
     crsurface = malloc (sizeof (cairo_glitz_surface_t));
     if (crsurface == NULL) {
-	_cairo_error (CAIRO_STATUS_NO_MEMORY);
+	_cairo_error_throw (CAIRO_STATUS_NO_MEMORY);
 	return (cairo_surface_t*) &_cairo_surface_nil;
     }
 
