@@ -34,6 +34,7 @@
  */
 
 #include "cairoint.h"
+#include "cairo-type1-private.h"
 #include "cairo-scaled-font-subsets-private.h"
 #include "cairo-output-stream-private.h"
 
@@ -45,6 +46,8 @@
 #include FT_OUTLINE_H
 #include FT_TYPE1_TABLES_H
 
+#include <ctype.h>
+
 typedef struct _cairo_type1_font_subset {
 
     cairo_scaled_font_subset_t *scaled_font_subset;
@@ -96,16 +99,16 @@ typedef struct _cairo_type1_font_subset {
     unsigned short eexec_key;
     cairo_bool_t hex_encode;
     int hex_column;
-
-    cairo_status_t status;
 } cairo_type1_font_subset_t;
 
 
 static cairo_status_t
 _cairo_type1_font_subset_create (cairo_unscaled_font_t      *unscaled_font,
-				 cairo_type1_font_subset_t **subset_return)
+				 cairo_type1_font_subset_t **subset_return,
+                                 cairo_bool_t                hex_encode)
 {
     cairo_ft_unscaled_font_t *ft_unscaled_font;
+    cairo_status_t status;
     FT_Face face;
     PS_FontInfoRec font_info;
     cairo_type1_font_subset_t *font;
@@ -114,13 +117,27 @@ _cairo_type1_font_subset_create (cairo_unscaled_font_t      *unscaled_font,
     ft_unscaled_font = (cairo_ft_unscaled_font_t *) unscaled_font;
 
     face = _cairo_ft_unscaled_font_lock_face (ft_unscaled_font);
+    if (face == NULL)
+	return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 
-    if (FT_Get_PS_Font_Info(face, &font_info) != 0)
-	return CAIRO_INT_STATUS_UNSUPPORTED;
+    if (FT_Get_PS_Font_Info(face, &font_info) != 0) {
+	status = CAIRO_INT_STATUS_UNSUPPORTED;
+        goto fail1;
+    }
+
+    /* OpenType/CFF fonts also have a PS_FontInfoRec */
+#if HAVE_FT_LOAD_SFNT_TABLE
+    if (FT_IS_SFNT (face)) {
+	status = CAIRO_INT_STATUS_UNSUPPORTED;
+        goto fail1;
+    }
+#endif
 
     font = calloc (sizeof (cairo_type1_font_subset_t), 1);
-    if (font == NULL)
-	return CAIRO_STATUS_NO_MEMORY;
+    if (font == NULL) {
+	status = _cairo_error (CAIRO_STATUS_NO_MEMORY);
+        goto fail1;
+    }
 
     font->base.unscaled_font = _cairo_unscaled_font_reference (unscaled_font);
     font->base.num_glyphs = face->num_glyphs;
@@ -131,8 +148,10 @@ _cairo_type1_font_subset_create (cairo_unscaled_font_t      *unscaled_font,
     font->base.ascent = face->ascender;
     font->base.descent = face->descender;
     font->base.base_font = strdup (face->family_name);
-    if (font->base.base_font == NULL)
-	goto fail1;
+    if (font->base.base_font == NULL) {
+        status = _cairo_error (CAIRO_STATUS_NO_MEMORY);
+	goto fail2;
+    }
 
     for (i = 0, j = 0; font->base.base_font[j]; j++) {
 	if (font->base.base_font[j] == ' ')
@@ -142,9 +161,12 @@ _cairo_type1_font_subset_create (cairo_unscaled_font_t      *unscaled_font,
     font->base.base_font[i] = '\0';
 
     font->glyphs = calloc (face->num_glyphs, sizeof font->glyphs[0]);
-    if (font->glyphs == NULL)
-	goto fail2;
+    if (font->glyphs == NULL) {
+        status = _cairo_error (CAIRO_STATUS_NO_MEMORY);
+	goto fail3;
+    }
 
+    font->hex_encode = hex_encode;
     font->num_glyphs = 0;
     for (i = 0; i < face->num_glyphs; i++)
 	font->glyphs[i].subset_index = -1;
@@ -157,12 +179,15 @@ _cairo_type1_font_subset_create (cairo_unscaled_font_t      *unscaled_font,
 
     return CAIRO_STATUS_SUCCESS;
 
- fail2:
+ fail3:
     free (font->base.base_font);
- fail1:
+ fail2:
+    _cairo_unscaled_font_destroy (unscaled_font);
     free (font);
+ fail1:
+    _cairo_ft_unscaled_font_unlock_face (ft_unscaled_font);
 
-    return CAIRO_STATUS_NO_MEMORY;
+    return status;
 }
 
 static int
@@ -177,15 +202,10 @@ cairo_type1_font_subset_use_glyph (cairo_type1_font_subset_t *font, int glyph)
     return font->glyphs[glyph].subset_index;
 }
 
-/* Magic constants for the type1 eexec encryption */
-static const unsigned short c1 = 52845, c2 = 22719;
-static const unsigned short private_dict_key = 55665;
-static const unsigned short charstring_key = 4330;
-
 static cairo_bool_t
 is_ps_delimiter(int c)
 {
-    const static char delimiters[] = "()[]{}<>/% \t\r\n";
+    static const char delimiters[] = "()[]{}<>/% \t\r\n";
 
     return strchr (delimiters, c) != NULL;
 }
@@ -196,6 +216,9 @@ find_token (const char *buffer, const char *end, const char *token)
     int i, length;
     /* FIXME: find substring really must be find_token */
 
+    if (buffer == NULL)
+	return NULL;
+
     length = strlen (token);
     for (i = 0; buffer + i < end - length + 1; i++)
 	if (memcmp (buffer + i, token, length) == 0)
@@ -235,7 +258,7 @@ cairo_type1_font_subset_find_segments (cairo_type1_font_subset_t *font)
     } else {
 	eexec_token = find_token ((char *) p, font->type1_end, "eexec");
 	if (eexec_token == NULL)
-	    return font->status = CAIRO_INT_STATUS_UNSUPPORTED;
+	    return CAIRO_INT_STATUS_UNSUPPORTED;
 
 	font->header_segment_size = eexec_token - (char *) p + strlen ("eexec\n");
 	font->header_segment = (char *) p;
@@ -258,7 +281,7 @@ cairo_type1_font_subset_write_header (cairo_type1_font_subset_t *font,
 
     start = find_token (font->header_segment, segment_end, "/FontName");
     if (start == NULL)
-	return font->status = CAIRO_INT_STATUS_UNSUPPORTED;
+	return CAIRO_INT_STATUS_UNSUPPORTED;
 
     _cairo_output_stream_write (font->output, font->header_segment,
 				start - font->header_segment);
@@ -267,18 +290,18 @@ cairo_type1_font_subset_write_header (cairo_type1_font_subset_t *font,
 
     end = find_token (start, segment_end, "def");
     if (end == NULL)
-	return font->status = CAIRO_INT_STATUS_UNSUPPORTED;
+	return CAIRO_INT_STATUS_UNSUPPORTED;
     end += 3;
 
     start = find_token (end, segment_end, "/Encoding");
     if (start == NULL)
-	return font->status = CAIRO_INT_STATUS_UNSUPPORTED;
+	return CAIRO_INT_STATUS_UNSUPPORTED;
     _cairo_output_stream_write (font->output, end, start - end);
 
     _cairo_output_stream_printf (font->output,
 				 "/Encoding 256 array\n"
 				 "0 1 255 {1 index exch /.notdef put} for\n");
-    for (i = 0; i < font->base.num_glyphs; i++) {
+    for (i = 1; i < font->base.num_glyphs; i++) {
 	if (font->glyphs[i].subset_index < 0)
 	    continue;
 	_cairo_output_stream_printf (font->output,
@@ -290,12 +313,12 @@ cairo_type1_font_subset_write_header (cairo_type1_font_subset_t *font,
 
     end = find_token (start, segment_end, "def");
     if (end == NULL)
-	return font->status = CAIRO_INT_STATUS_UNSUPPORTED;
+	return CAIRO_INT_STATUS_UNSUPPORTED;
     end += 3;
 
     _cairo_output_stream_write (font->output, end, segment_end - end);
 
-    return font->status;
+    return font->output->status;
 }
 
 static int
@@ -309,7 +332,7 @@ hex_to_int (int ch)
 	return ch - 'a' + 10;
 }
 
-static void
+static cairo_status_t
 cairo_type1_font_subset_write_encrypted (cairo_type1_font_subset_t *font,
 					 const char *data, unsigned int length)
 {
@@ -323,7 +346,7 @@ cairo_type1_font_subset_write_encrypted (cairo_type1_font_subset_t *font,
     while (in < end) {
 	p = *in++;
 	c = p ^ (font->eexec_key >> 8);
-	font->eexec_key = (c + font->eexec_key) * c1 + c2;
+	font->eexec_key = (c + font->eexec_key) * CAIRO_TYPE1_ENCRYPT_C1 + CAIRO_TYPE1_ENCRYPT_C2;
 
 	if (font->hex_encode) {
 	    digits[0] = hex_digits[c >> 4];
@@ -342,12 +365,14 @@ cairo_type1_font_subset_write_encrypted (cairo_type1_font_subset_t *font,
 	    _cairo_output_stream_write (font->output, digits, 1);
 	}
     }
+
+    return font->output->status;
 }
 
 static cairo_status_t
 cairo_type1_font_subset_decrypt_eexec_segment (cairo_type1_font_subset_t *font)
 {
-    unsigned short r = private_dict_key;
+    unsigned short r = CAIRO_TYPE1_PRIVATE_DICT_KEY;
     unsigned char *in, *end;
     char *out;
     int c, p;
@@ -357,9 +382,9 @@ cairo_type1_font_subset_decrypt_eexec_segment (cairo_type1_font_subset_t *font)
 
     font->cleartext = malloc (font->eexec_segment_size);
     if (font->cleartext == NULL)
-	return font->status = CAIRO_STATUS_NO_MEMORY;
-    out = font->cleartext;
+	return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 
+    out = font->cleartext;
     while (in < end) {
 	if (font->eexec_segment_is_ascii) {
 	    c = *in++;
@@ -370,14 +395,13 @@ cairo_type1_font_subset_decrypt_eexec_segment (cairo_type1_font_subset_t *font)
 	    c = *in++;
 	}
 	p = c ^ (r >> 8);
-	r = (c + r) * c1 + c2;
+	r = (c + r) * CAIRO_TYPE1_ENCRYPT_C1 + CAIRO_TYPE1_ENCRYPT_C2;
 
 	*out++ = p;
     }
-
     font->cleartext_end = out;
 
-    return font->status;
+    return CAIRO_STATUS_SUCCESS;
 }
 
 static const char *
@@ -428,7 +452,7 @@ cairo_type1_font_subset_get_glyph_names_and_widths (cairo_type1_font_subset_t *f
 			       FT_LOAD_NO_BITMAP | FT_LOAD_IGNORE_TRANSFORM);
 	if (error != 0) {
 	    printf ("could not load glyph %d\n", i);
-	    return font->status = CAIRO_STATUS_NO_MEMORY;
+	    return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 	}
 
 	font->glyphs[i].width = font->face->glyph->metrics.horiAdvance;
@@ -436,12 +460,12 @@ cairo_type1_font_subset_get_glyph_names_and_widths (cairo_type1_font_subset_t *f
 	error = FT_Get_Glyph_Name(font->face, i, buffer, sizeof buffer);
 	if (error != 0) {
 	    printf ("could not get glyph name for glyph %d\n", i);
-	    return font->status = CAIRO_STATUS_NO_MEMORY;
+	    return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 	}
 
 	font->glyphs[i].name = strdup (buffer);
 	if (font->glyphs[i].name == NULL)
-	    return font->status = CAIRO_STATUS_NO_MEMORY;
+	    return _cairo_error (CAIRO_STATUS_NO_MEMORY);
     }
 
     return CAIRO_STATUS_SUCCESS;
@@ -450,13 +474,13 @@ cairo_type1_font_subset_get_glyph_names_and_widths (cairo_type1_font_subset_t *f
 static void
 cairo_type1_font_subset_decrypt_charstring (const unsigned char *in, int size, unsigned char *out)
 {
-    unsigned short r = charstring_key;
+    unsigned short r = CAIRO_TYPE1_CHARSTRING_KEY;
     int c, p, i;
 
     for (i = 0; i < size; i++) {
         c = *in++;
 	p = c ^ (r >> 8);
-	r = (c + r) * c1 + c2;
+	r = (c + r) * CAIRO_TYPE1_ENCRYPT_C1 + CAIRO_TYPE1_ENCRYPT_C2;
 	*out++ = p;
     }
 }
@@ -480,7 +504,12 @@ cairo_type1_font_subset_decode_integer (const unsigned char *p, int *integer)
     return p;
 }
 
-static const char *ps_standard_encoding[256] = {
+#if 0
+/*
+ * The two tables that follow are generated using this perl code:
+ */
+
+@encoding = (
 	/*   0 */
 	NULL,		NULL,		NULL,		NULL,
 	NULL,		NULL,		NULL,		NULL,
@@ -561,37 +590,138 @@ static const char *ps_standard_encoding[256] = {
 	NULL,		"dotlessi",	NULL,		NULL,
 	"lslash",	"oslash",	"oe",		"germandbls",
 	NULL,		NULL,		NULL,		NULL
+	);
+
+print "static const char ps_standard_encoding_symbol[] = {\n";
+$s = qq( "\\0");
+for $sym (@encoding) {
+    if (! ($sym eq NULL)) {
+        $ss = qq( "$sym\\0");
+	if (length($s) + length($ss) > 78) {
+	  print qq( $s\n);
+	  $s = "";
+	}
+	$s .= $ss;
+    }
+}
+print qq( $s\n);
+print "};\n\n";
+print "static const int16_t ps_standard_encoding_offset[256] = {\n";
+$offset = 1;
+$s = qq();
+for $sym (@encoding) {
+    if (! ($sym eq NULL)) {
+	$ss = qq( $offset/*$sym*/,);
+	$offset += length($sym) + 1;
+    } else {
+	$ss = qq( 0,);
+    }
+    if (length($s) + length($ss) > 78) {
+      print qq( $s\n);
+      $s = "";
+    }
+    $s .= $ss;
+}
+print qq( $s\n);
+print "};\n";
+exit;
+#endif
+
+static const char ps_standard_encoding_symbol[] = {
+  "\0" "space\0" "exclam\0" "quotedbl\0" "numbersign\0" "dollar\0" "percent\0"
+  "ampersand\0" "quoteright\0" "parenleft\0" "parenright\0" "asterisk\0"
+  "plus\0" "comma\0" "hyphen\0" "period\0" "slash\0" "zero\0" "one\0" "two\0"
+  "three\0" "four\0" "five\0" "six\0" "seven\0" "eight\0" "nine\0" "colon\0"
+  "semicolon\0" "less\0" "equal\0" "greater\0" "question\0" "at\0" "A\0" "B\0"
+  "C\0" "D\0" "E\0" "F\0" "G\0" "H\0" "I\0" "J\0" "K\0" "L\0" "M\0" "N\0" "O\0"
+  "P\0" "Q\0" "R\0" "S\0" "T\0" "U\0" "V\0" "W\0" "X\0" "Y\0" "Z\0"
+  "bracketleft\0" "backslash\0" "bracketright\0" "asciicircum\0" "underscore\0"
+  "quoteleft\0" "a\0" "b\0" "c\0" "d\0" "e\0" "f\0" "g\0" "h\0" "i\0" "j\0"
+  "k\0" "l\0" "m\0" "n\0" "o\0" "p\0" "q\0" "r\0" "s\0" "t\0" "u\0" "v\0" "w\0"
+  "x\0" "y\0" "z\0" "braceleft\0" "bar\0" "braceright\0" "asciitilde\0"
+  "exclamdown\0" "cent\0" "sterling\0" "fraction\0" "yen\0" "florin\0"
+  "section\0" "currency\0" "quotesingle\0" "quotedblleft\0" "guillemotleft\0"
+  "guilsinglleft\0" "guilsinglright\0" "fi\0" "fl\0" "endash\0" "dagger\0"
+  "daggerdbl\0" "periodcentered\0" "paragraph\0" "bullet\0" "quotesinglbase\0"
+  "quotedblbase\0" "quotedblright\0" "guillemotright\0" "ellipsis\0"
+  "perthousand\0" "questiondown\0" "grave\0" "acute\0" "circumflex\0" "tilde\0"
+  "macron\0" "breve\0" "dotaccent\0" "dieresis\0" "ring\0" "cedilla\0"
+  "hungarumlaut\0" "ogonek\0" "caron\0" "emdash\0" "AE\0" "ordfeminine\0"
+  "Lslash\0" "Oslash\0" "OE\0" "ordmasculine\0" "ae\0" "dotlessi\0" "lslash\0"
+  "oslash\0" "oe\0" "germandbls\0"
 };
 
-static void
+static const int16_t ps_standard_encoding_offset[256] = {
+  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 1/*space*/, 7/*exclam*/, 14/*quotedbl*/, 23/*numbersign*/,
+  34/*dollar*/, 41/*percent*/, 49/*ampersand*/, 59/*quoteright*/,
+  70/*parenleft*/, 80/*parenright*/, 91/*asterisk*/, 100/*plus*/, 105/*comma*/,
+  111/*hyphen*/, 118/*period*/, 125/*slash*/, 131/*zero*/, 136/*one*/,
+  140/*two*/, 144/*three*/, 150/*four*/, 155/*five*/, 160/*six*/, 164/*seven*/,
+  170/*eight*/, 176/*nine*/, 181/*colon*/, 187/*semicolon*/, 197/*less*/,
+  202/*equal*/, 208/*greater*/, 216/*question*/, 225/*at*/, 228/*A*/, 230/*B*/,
+  232/*C*/, 234/*D*/, 236/*E*/, 238/*F*/, 240/*G*/, 242/*H*/, 244/*I*/,
+  246/*J*/, 248/*K*/, 250/*L*/, 252/*M*/, 254/*N*/, 256/*O*/, 258/*P*/,
+  260/*Q*/, 262/*R*/, 264/*S*/, 266/*T*/, 268/*U*/, 270/*V*/, 272/*W*/,
+  274/*X*/, 276/*Y*/, 278/*Z*/, 280/*bracketleft*/, 292/*backslash*/,
+  302/*bracketright*/, 315/*asciicircum*/, 327/*underscore*/, 338/*quoteleft*/,
+  348/*a*/, 350/*b*/, 352/*c*/, 354/*d*/, 356/*e*/, 358/*f*/, 360/*g*/,
+  362/*h*/, 364/*i*/, 366/*j*/, 368/*k*/, 370/*l*/, 372/*m*/, 374/*n*/,
+  376/*o*/, 378/*p*/, 380/*q*/, 382/*r*/, 384/*s*/, 386/*t*/, 388/*u*/,
+  390/*v*/, 392/*w*/, 394/*x*/, 396/*y*/, 398/*z*/, 400/*braceleft*/,
+  410/*bar*/, 414/*braceright*/, 425/*asciitilde*/, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+  436/*exclamdown*/, 447/*cent*/, 452/*sterling*/, 461/*fraction*/, 470/*yen*/,
+  474/*florin*/, 481/*section*/, 489/*currency*/, 498/*quotesingle*/,
+  510/*quotedblleft*/, 523/*guillemotleft*/, 537/*guilsinglleft*/,
+  551/*guilsinglright*/, 566/*fi*/, 569/*fl*/, 0, 572/*endash*/, 579/*dagger*/,
+  586/*daggerdbl*/, 596/*periodcentered*/, 0, 611/*paragraph*/, 621/*bullet*/,
+  628/*quotesinglbase*/, 643/*quotedblbase*/, 656/*quotedblright*/,
+  670/*guillemotright*/, 685/*ellipsis*/, 694/*perthousand*/, 0,
+  706/*questiondown*/, 0, 719/*grave*/, 725/*acute*/, 731/*circumflex*/,
+  742/*tilde*/, 748/*macron*/, 755/*breve*/, 761/*dotaccent*/, 771/*dieresis*/,
+  0, 780/*ring*/, 785/*cedilla*/, 0, 793/*hungarumlaut*/, 806/*ogonek*/,
+  813/*caron*/, 819/*emdash*/, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+  826/*AE*/, 0, 829/*ordfeminine*/, 0, 0, 0, 0, 841/*Lslash*/, 848/*Oslash*/,
+  855/*OE*/, 858/*ordmasculine*/, 0, 0, 0, 0, 0, 871/*ae*/, 0, 0, 0,
+  874/*dotlessi*/, 0, 0, 883/*lslash*/, 890/*oslash*/, 897/*oe*/,
+  900/*germandbls*/, 0, 0, 0, 0,
+};
+
+#define ps_standard_encoding(index) ((index) ? ps_standard_encoding_symbol+ps_standard_encoding_offset[(index)] : NULL)
+
+static cairo_status_t
 use_standard_encoding_glyph (cairo_type1_font_subset_t *font, int index)
 {
     const char *glyph_name;
 
     if (index < 0 || index > 255)
-	return;
+	return CAIRO_STATUS_SUCCESS;
 
-    glyph_name = ps_standard_encoding[index];
+    glyph_name = ps_standard_encoding(index);
     if (glyph_name == NULL)
-	return;
+	return CAIRO_STATUS_SUCCESS;
 
     index = cairo_type1_font_subset_lookup_glyph (font,
 						  glyph_name,
 						  strlen(glyph_name));
     if (index < 0)
-	return;
+	return CAIRO_INT_STATUS_UNSUPPORTED;
 
     cairo_type1_font_subset_use_glyph (font, index);
+
+    return CAIRO_STATUS_SUCCESS;
 }
 
 #define TYPE1_CHARSTRING_COMMAND_ESCAPE		(12)
 #define TYPE1_CHARSTRING_COMMAND_SEAC		(32 + 6)
 
-static void
+static cairo_status_t
 cairo_type1_font_subset_look_for_seac(cairo_type1_font_subset_t *font,
 				      const char *name, int name_length,
 				      const char *encrypted_charstring, int encrypted_charstring_length)
 {
+    cairo_status_t status;
     unsigned char *charstring;
     const unsigned char *end;
     const unsigned char *p;
@@ -600,7 +730,7 @@ cairo_type1_font_subset_look_for_seac(cairo_type1_font_subset_t *font,
 
     charstring = malloc (encrypted_charstring_length);
     if (charstring == NULL)
-	return;
+	return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 
     cairo_type1_font_subset_decrypt_charstring ((const unsigned char *)
 						encrypted_charstring,
@@ -626,8 +756,14 @@ cairo_type1_font_subset_look_for_seac(cairo_type1_font_subset_t *font,
 		 * glyph is composed from.  All we need to do is to
 		 * make sure those glyphs are present in the subset
 		 * under their standard names. */
-		use_standard_encoding_glyph (font, stack[3]);
-		use_standard_encoding_glyph (font, stack[4]);
+		status = use_standard_encoding_glyph (font, stack[3]);
+		if (status)
+		    return status;
+
+		status = use_standard_encoding_glyph (font, stack[4]);
+		if (status)
+		    return status;
+
 		sp = 0;
 		break;
 
@@ -644,35 +780,50 @@ cairo_type1_font_subset_look_for_seac(cairo_type1_font_subset_t *font,
     }
 
     free (charstring);
+
+    return CAIRO_STATUS_SUCCESS;
 }
 
-static void
+static cairo_status_t
 write_used_glyphs (cairo_type1_font_subset_t *font,
 		   const char *name, int name_length,
 		   const char *charstring, int charstring_length)
 {
+    cairo_status_t status;
     char buffer[256];
     int length;
 
     length = snprintf (buffer, sizeof buffer,
 		       "/%.*s %d %s ",
 		       name_length, name, charstring_length, font->rd);
-    cairo_type1_font_subset_write_encrypted (font, buffer, length);
-    cairo_type1_font_subset_write_encrypted (font,
-					     charstring, charstring_length);
+    status = cairo_type1_font_subset_write_encrypted (font, buffer, length);
+    if (status)
+	return status;
+
+    status = cairo_type1_font_subset_write_encrypted (font,
+					              charstring,
+						      charstring_length);
+    if (status)
+	return status;
+
     length = snprintf (buffer, sizeof buffer, "%s\n", font->nd);
-    cairo_type1_font_subset_write_encrypted (font, buffer, length);
+    status = cairo_type1_font_subset_write_encrypted (font, buffer, length);
+    if (status)
+	return status;
+
+    return CAIRO_STATUS_SUCCESS;
 }
 
-typedef void (*glyph_func_t) (cairo_type1_font_subset_t *font,
-			      const char *name, int name_length,
-			      const char *charstring, int charstring_length);
+typedef cairo_status_t (*glyph_func_t) (cairo_type1_font_subset_t *font,
+			                const char *name, int name_length,
+			                const char *charstring, int charstring_length);
 
-static const char *
+static cairo_status_t
 cairo_type1_font_subset_for_each_glyph (cairo_type1_font_subset_t *font,
 					const char *dict_start,
 					const char *dict_end,
-					glyph_func_t func)
+					glyph_func_t func,
+					const char **dict_out)
 {
     int charstring_length, name_length, glyph_index;
     const char *p, *charstring, *name;
@@ -700,10 +851,8 @@ cairo_type1_font_subset_for_each_glyph (cairo_type1_font_subset_t *font,
 	name_length = p - name;
 
 	charstring_length = strtol (p, &end, 10);
-	if (p == end) {
-	    font->status = CAIRO_INT_STATUS_UNSUPPORTED;
-	    return NULL;
-	}
+	if (p == end)
+	    return CAIRO_INT_STATUS_UNSUPPORTED;
 
 	/* Skip past -| or RD to binary data.  There is exactly one space
 	 * between the -| or RD token and the encrypted data, thus '+ 1'. */
@@ -716,25 +865,31 @@ cairo_type1_font_subset_for_each_glyph (cairo_type1_font_subset_t *font,
 
 	/* In case any of the skip_token() calls above reached EOF, p will
 	 * be equal to dict_end. */
-	if (p == dict_end) {
-	    font->status = CAIRO_INT_STATUS_UNSUPPORTED;
-	    return NULL;
-	}
+	if (p == dict_end)
+	    return CAIRO_INT_STATUS_UNSUPPORTED;
 
 	glyph_index = cairo_type1_font_subset_lookup_glyph (font,
 							    name, name_length);
-	if (font->glyphs[glyph_index].subset_index >= 0)
-	    func (font, name, name_length, charstring, charstring_length);
+	if (font->glyphs[glyph_index].subset_index >= 0) {
+	    cairo_status_t status = func (font,
+		                          name, name_length,
+					  charstring, charstring_length);
+	    if (status)
+		return status;
+	}
     }
 
-    return p;
+    *dict_out = p;
+
+    return CAIRO_STATUS_SUCCESS;
 }
 
 
-static const char *
+static cairo_status_t
 cairo_type1_font_subset_write_private_dict (cairo_type1_font_subset_t *font,
 					    const char                *name)
 {
+    cairo_status_t status;
     const char *p, *charstrings, *dict_start;
     const char *closefile_token;
     char buffer[32], *glyph_count_end;
@@ -757,88 +912,98 @@ cairo_type1_font_subset_write_private_dict (cairo_type1_font_subset_t *font,
      * conceivably have "/CharStrings" in it, so we might need to skip
      * this more cleverly. */
     charstrings = find_token (font->cleartext, font->cleartext_end, "/CharStrings");
-    if (charstrings == NULL) {
-	font->status = CAIRO_INT_STATUS_UNSUPPORTED;
-	return NULL;
-    }
+    if (charstrings == NULL)
+	return CAIRO_INT_STATUS_UNSUPPORTED;
 
     /* Scan past /CharStrings and the integer following it. */
     p = charstrings + strlen ("/CharStrings");
     num_charstrings = strtol (p, &glyph_count_end, 10);
-    if (p == glyph_count_end) {
-	font->status = CAIRO_INT_STATUS_UNSUPPORTED;
-	return NULL;
-    }
+    if (p == glyph_count_end)
+	return CAIRO_INT_STATUS_UNSUPPORTED;
 
     /* Look for a '/' which marks the beginning of the first glyph
      * definition. */
     for (p = glyph_count_end; p < font->cleartext_end; p++)
 	if (*p == '/')
 	    break;
-    if (p == font->cleartext_end) {
-	font->status = CAIRO_INT_STATUS_UNSUPPORTED;
-	return NULL;
-    }
+    if (p == font->cleartext_end)
+	return CAIRO_INT_STATUS_UNSUPPORTED;
     dict_start = p;
 
-    if (cairo_type1_font_subset_get_glyph_names_and_widths (font))
-	return NULL;
+    status = cairo_type1_font_subset_get_glyph_names_and_widths (font);
+    if (status)
+	return status;
 
     /* Now that we have the private dictionary broken down in
      * sections, do the first pass through the glyph definitions to
      * figure out which subrs and othersubrs are use and which extra
      * glyphs may be required by the seac operator. */
-    p = cairo_type1_font_subset_for_each_glyph (font,
-						dict_start,
-						font->cleartext_end,
-						cairo_type1_font_subset_look_for_seac);
+    status = cairo_type1_font_subset_for_each_glyph (font,
+						     dict_start,
+						     font->cleartext_end,
+						     cairo_type1_font_subset_look_for_seac,
+						     &p);
+    if (status)
+	return status;
 
     closefile_token = find_token (p, font->cleartext_end, "closefile");
-    if (closefile_token == NULL) {
-	font->status = CAIRO_INT_STATUS_UNSUPPORTED;
-	return NULL;
-    }
+    if (closefile_token == NULL)
+	return CAIRO_INT_STATUS_UNSUPPORTED;
 
-    if (cairo_type1_font_subset_get_glyph_names_and_widths (font))
-	return NULL;
+    status = cairo_type1_font_subset_get_glyph_names_and_widths (font);
+    if (status)
+	return status;
 
     /* We're ready to start outputting. First write the header,
      * i.e. the public part of the font dict.*/
-    if (cairo_type1_font_subset_write_header (font, name))
-	return NULL;
+    status = cairo_type1_font_subset_write_header (font, name);
+    if (status)
+	return status;
 
     font->base.header_size = _cairo_output_stream_get_position (font->output);
 
 
     /* Start outputting the private dict.  First output everything up
      * to the /CharStrings token. */
-    cairo_type1_font_subset_write_encrypted (font, font->cleartext,
-					     charstrings - font->cleartext);
+    status = cairo_type1_font_subset_write_encrypted (font, font->cleartext,
+					         charstrings - font->cleartext);
+    if (status)
+	return status;
 
     /* Write out new charstring count */
     length = snprintf (buffer, sizeof buffer,
 		       "/CharStrings %d", font->num_glyphs);
-    cairo_type1_font_subset_write_encrypted (font, buffer, length);
+    status = cairo_type1_font_subset_write_encrypted (font, buffer, length);
+    if (status)
+	return status;
 
     /* Write out text between the charstring count and the first
      * charstring definition */
-    cairo_type1_font_subset_write_encrypted (font, glyph_count_end,
-					     dict_start - glyph_count_end);
+    status = cairo_type1_font_subset_write_encrypted (font, glyph_count_end,
+	                                          dict_start - glyph_count_end);
+    if (status)
+	return status;
 
     /* Write out the charstring definitions for each of the glyphs in
      * the subset. */
-    p = cairo_type1_font_subset_for_each_glyph (font,
-						dict_start,
-						font->cleartext_end,
-						write_used_glyphs);
+    status = cairo_type1_font_subset_for_each_glyph (font,
+						     dict_start,
+						     font->cleartext_end,
+						     write_used_glyphs,
+						     &p);
+    if (status)
+	return status;
 
     /* Output what's left between the end of the glyph definitions and
      * the end of the private dict to the output. */
-    cairo_type1_font_subset_write_encrypted (font, p,
-					     closefile_token - p + strlen ("closefile") + 1);
+    status = cairo_type1_font_subset_write_encrypted (font, p,
+	                        closefile_token - p + strlen ("closefile") + 1);
+    if (status)
+	return status;
+
     _cairo_output_stream_write (font->output, "\n", 1);
 
-    return p;
+    return CAIRO_STATUS_SUCCESS;
 }
 
 static cairo_status_t
@@ -858,12 +1023,12 @@ cairo_type1_font_subset_write_trailer(cairo_type1_font_subset_t *font)
 
     cleartomark_token = find_token (font->type1_data, font->type1_end, "cleartomark");
     if (cleartomark_token == NULL)
-	return font->status = CAIRO_INT_STATUS_UNSUPPORTED;
+	return CAIRO_INT_STATUS_UNSUPPORTED;
 
     _cairo_output_stream_write (font->output, cleartomark_token,
 				font->type1_end - cleartomark_token);
 
-    return font->status;
+    return CAIRO_STATUS_SUCCESS;
 }
 
 static cairo_status_t
@@ -871,21 +1036,22 @@ type1_font_write (void *closure, const unsigned char *data, unsigned int length)
 {
     cairo_type1_font_subset_t *font = closure;
 
-    font->status =
-	_cairo_array_append_multiple (&font->contents, data, length);
-
-    return font->status;
+    return _cairo_array_append_multiple (&font->contents, data, length);
 }
 
 static cairo_status_t
 cairo_type1_font_subset_write (cairo_type1_font_subset_t *font,
 			       const char *name)
 {
-    if (cairo_type1_font_subset_find_segments (font))
-	return font->status;
+    cairo_status_t status;
 
-    if (cairo_type1_font_subset_decrypt_eexec_segment (font))
-	return font->status;
+    status = cairo_type1_font_subset_find_segments (font);
+    if (status)
+	return status;
+
+    status = cairo_type1_font_subset_decrypt_eexec_segment (font);
+    if (status)
+	return status;
 
     /* Determine which glyph definition delimiters to use. */
     if (find_token (font->cleartext, font->cleartext_end, "/-|") != NULL) {
@@ -896,25 +1062,28 @@ cairo_type1_font_subset_write (cairo_type1_font_subset_t *font,
 	font->nd = "ND";
     } else {
 	/* Don't know *what* kind of font this is... */
-	return font->status = CAIRO_INT_STATUS_UNSUPPORTED;
+	return CAIRO_INT_STATUS_UNSUPPORTED;
     }
 
-    font->eexec_key = private_dict_key;
-    font->hex_encode = TRUE;
+    font->eexec_key = CAIRO_TYPE1_PRIVATE_DICT_KEY;
     font->hex_column = 0;
 
-    cairo_type1_font_subset_write_private_dict (font, name);
+    status = cairo_type1_font_subset_write_private_dict (font, name);
+    if (status)
+	return status;
 
     font->base.data_size = _cairo_output_stream_get_position (font->output) -
 	font->base.header_size;
 
-    cairo_type1_font_subset_write_trailer (font);
+    status = cairo_type1_font_subset_write_trailer (font);
+    if (status)
+	return status;
 
     font->base.trailer_size =
 	_cairo_output_stream_get_position (font->output) -
 	font->base.header_size - font->base.data_size;
 
-    return font->status;
+    return CAIRO_STATUS_SUCCESS;
 }
 
 static cairo_status_t
@@ -925,45 +1094,52 @@ cairo_type1_font_subset_generate (void       *abstract_font,
     cairo_type1_font_subset_t *font = abstract_font;
     cairo_ft_unscaled_font_t *ft_unscaled_font;
     unsigned long ret;
+    cairo_status_t status;
 
     ft_unscaled_font = (cairo_ft_unscaled_font_t *) font->base.unscaled_font;
     font->face = _cairo_ft_unscaled_font_lock_face (ft_unscaled_font);
-
-    /* If anything fails below, it's out of memory. */
-    font->status = CAIRO_STATUS_NO_MEMORY;
+    if (font->face == NULL)
+	return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 
     font->type1_length = font->face->stream->size;
     font->type1_data = malloc (font->type1_length);
-    if (font->type1_data == NULL)
+    if (font->type1_data == NULL) {
+	status = _cairo_error (CAIRO_STATUS_NO_MEMORY);
 	goto fail;
+    }
 
     if (font->face->stream->read) {
 	ret = font->face->stream->read (font->face->stream, 0,
 					(unsigned char *) font->type1_data,
 					font->type1_length);
-	if (ret != font->type1_length)
+	if (ret != font->type1_length) {
+	    status = _cairo_error (CAIRO_STATUS_READ_ERROR);
 	    goto fail;
+	}
     } else {
 	memcpy (font->type1_data,
 		font->face->stream->base, font->type1_length);
     }
 
-    if (_cairo_array_grow_by (&font->contents, 4096) != CAIRO_STATUS_SUCCESS)
+    status = _cairo_array_grow_by (&font->contents, 4096);
+    if (status)
 	goto fail;
 
     font->output = _cairo_output_stream_create (type1_font_write, NULL, font);
-    if (font->output == NULL)
+    status = _cairo_output_stream_get_status (font->output);
+    if (status)
 	goto fail;
 
-    font->status = CAIRO_STATUS_SUCCESS;
-    cairo_type1_font_subset_write (font, name);
+    status = cairo_type1_font_subset_write (font, name);
+    if (status)
+	goto fail;
 
     font->base.data = _cairo_array_index (&font->contents, 0);
 
  fail:
     _cairo_ft_unscaled_font_unlock_face (ft_unscaled_font);
 
-    return font->status;
+    return status;
 }
 
 static void
@@ -994,9 +1170,10 @@ cairo_type1_font_subset_destroy (void *abstract_font)
 cairo_status_t
 _cairo_type1_subset_init (cairo_type1_subset_t		*type1_subset,
 			  const char			*name,
-			  cairo_scaled_font_subset_t	*scaled_font_subset)
+			  cairo_scaled_font_subset_t	*scaled_font_subset,
+                          cairo_bool_t                   hex_encode)
 {
-    cairo_type1_font_subset_t *font;
+    cairo_type1_font_subset_t *font = NULL; /* hide compiler warning */
     cairo_status_t status;
     unsigned long parent_glyph, length;
     unsigned int i;
@@ -1006,9 +1183,12 @@ _cairo_type1_subset_init (cairo_type1_subset_t		*type1_subset,
     if (!_cairo_scaled_font_is_ft (scaled_font_subset->scaled_font))
 	return CAIRO_INT_STATUS_UNSUPPORTED;
 
+    if (_cairo_ft_scaled_font_is_vertical (scaled_font_subset->scaled_font))
+        return CAIRO_INT_STATUS_UNSUPPORTED;
+
     unscaled_font = _cairo_ft_scaled_font_get_unscaled_font (scaled_font_subset->scaled_font);
 
-    status = _cairo_type1_font_subset_create (unscaled_font, &font);
+    status = _cairo_type1_font_subset_create (unscaled_font, &font, hex_encode);
     if (status)
 	return status;
 
@@ -1017,9 +1197,6 @@ _cairo_type1_subset_init (cairo_type1_subset_t		*type1_subset,
 	cairo_type1_font_subset_use_glyph (font, parent_glyph);
     }
 
-    /* Pull in the .notdef glyph */
-    cairo_type1_font_subset_use_glyph (font, 0);
-
     status = cairo_type1_font_subset_generate (font, name);
     if (status)
 	goto fail1;
@@ -1080,3 +1257,31 @@ _cairo_type1_subset_fini (cairo_type1_subset_t *subset)
     free (subset->data);
 }
 
+cairo_bool_t
+_cairo_type1_scaled_font_is_type1 (cairo_scaled_font_t *scaled_font)
+{
+    cairo_ft_unscaled_font_t *unscaled;
+    FT_Face face;
+    PS_FontInfoRec font_info;
+    cairo_bool_t is_type1 = FALSE;
+
+    if (!_cairo_scaled_font_is_ft (scaled_font))
+       return FALSE;
+    unscaled = (cairo_ft_unscaled_font_t *) _cairo_ft_scaled_font_get_unscaled_font (scaled_font);
+    face = _cairo_ft_unscaled_font_lock_face (unscaled);
+    if (!face)
+        return FALSE;
+
+    if (FT_Get_PS_Font_Info(face, &font_info) == 0)
+        is_type1 = TRUE;
+
+    /* OpenType/CFF fonts also have a PS_FontInfoRec */
+#if HAVE_FT_LOAD_SFNT_TABLE
+    if (FT_IS_SFNT (face))
+        is_type1 = FALSE;
+#endif
+
+    _cairo_ft_unscaled_font_unlock_face (unscaled);
+
+    return is_type1;
+}
