@@ -105,10 +105,10 @@ _cairo_win32_print_gdi_error (const char *context)
      * is no CAIRO_STATUS_UNKNOWN_ERROR.
      */
 
-    return CAIRO_STATUS_NO_MEMORY;
+    return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 }
 
-static uint32_t
+uint32_t
 _cairo_win32_flags_for_dc (HDC dc)
 {
     uint32_t flags = 0;
@@ -151,7 +151,7 @@ _create_dc_and_bitmap (cairo_win32_surface_t *surface,
 		       cairo_format_t         format,
 		       int                    width,
 		       int                    height,
-		       char                 **bits_out,
+		       unsigned char        **bits_out,
 		       int                   *rowstride_out)
 {
     cairo_status_t status;
@@ -186,9 +186,9 @@ _create_dc_and_bitmap (cairo_win32_surface_t *surface,
     }
 
     if (num_palette > 2) {
-	bitmap_info = malloc (sizeof (BITMAPINFOHEADER) + num_palette * sizeof (RGBQUAD));
+	bitmap_info = _cairo_malloc_ab_plus_c (num_palette, sizeof(RGBQUAD), sizeof(BITMAPINFOHEADER));
 	if (!bitmap_info)
-	    return CAIRO_STATUS_NO_MEMORY;
+	    return _cairo_error (CAIRO_STATUS_NO_MEMORY);
     } else {
 	bitmap_info = (BITMAPINFO *)&bmi_stack;
     }
@@ -329,12 +329,12 @@ _cairo_win32_surface_create_for_dc (HDC             original_dc,
 {
     cairo_status_t status;
     cairo_win32_surface_t *surface;
-    char *bits;
+    unsigned char *bits;
     int rowstride;
 
     surface = malloc (sizeof (cairo_win32_surface_t));
     if (surface == NULL) {
-	_cairo_error (CAIRO_STATUS_NO_MEMORY);
+	_cairo_error_throw (CAIRO_STATUS_NO_MEMORY);
 	return NIL_SURFACE;
     }
 
@@ -347,7 +347,7 @@ _cairo_win32_surface_create_for_dc (HDC             original_dc,
     surface->image = cairo_image_surface_create_for_data (bits, format,
 							  width, height, rowstride);
     if (surface->image->status) {
-	status = CAIRO_STATUS_NO_MEMORY;
+	status = _cairo_error (CAIRO_STATUS_NO_MEMORY);
 	goto FAIL;
     }
 
@@ -358,11 +358,7 @@ _cairo_win32_surface_create_for_dc (HDC             original_dc,
     surface->clip_rect.width = width;
     surface->clip_rect.height = height;
 
-    surface->saved_clip = CreateRectRgn (0, 0, 0, 0);
-    if (GetClipRgn (surface->dc, surface->saved_clip) == 0) {
-	DeleteObject(surface->saved_clip);
-	surface->saved_clip = NULL;
-    }
+    surface->saved_clip = NULL;
 
     surface->extents = surface->clip_rect;
 
@@ -380,13 +376,7 @@ _cairo_win32_surface_create_for_dc (HDC             original_dc,
     if (surface)
 	free (surface);
 
-    if (status == CAIRO_STATUS_NO_MEMORY) {
-	_cairo_error (CAIRO_STATUS_NO_MEMORY);
-	return NIL_SURFACE;
-    } else {
-	_cairo_error (status);
-	return NIL_SURFACE;
-    }
+    return NIL_SURFACE;
 }
 
 static cairo_surface_t *
@@ -409,13 +399,9 @@ _cairo_win32_surface_create_similar_internal (void	    *abstract_src,
 	/* otherwise, create a ddb */
 	HBITMAP ddb = CreateCompatibleBitmap (src->dc, width, height);
 	HDC ddb_dc = CreateCompatibleDC (src->dc);
-	HRGN crgn = CreateRectRgn (0, 0, width, height);
 	HBITMAP saved_dc_bitmap;
 
 	saved_dc_bitmap = SelectObject (ddb_dc, ddb);
-	SelectClipRgn (ddb_dc, crgn);
-
-	DeleteObject (crgn);
 
 	new_surf = (cairo_win32_surface_t*) cairo_win32_surface_create (ddb_dc);
 	new_surf->bitmap = ddb;
@@ -426,7 +412,7 @@ _cairo_win32_surface_create_similar_internal (void	    *abstract_src,
     return (cairo_surface_t*) new_surf;
 }
 
-static cairo_surface_t *
+cairo_surface_t *
 _cairo_win32_surface_create_similar (void	    *abstract_src,
 				     cairo_content_t content,
 				     int	     width,
@@ -435,7 +421,50 @@ _cairo_win32_surface_create_similar (void	    *abstract_src,
     return _cairo_win32_surface_create_similar_internal (abstract_src, content, width, height, FALSE);
 }
 
-static cairo_status_t
+cairo_status_t
+_cairo_win32_surface_clone_similar (void *abstract_surface,
+				    cairo_surface_t *src,
+				    int src_x,
+				    int src_y,
+				    int width,
+				    int height,
+				    cairo_surface_t **clone_out)
+{
+    cairo_content_t src_content;
+    cairo_surface_t *new_surface;
+    cairo_status_t status;
+    cairo_pattern_union_t pattern;
+
+    src_content = cairo_surface_get_content(src);
+    new_surface =
+	_cairo_win32_surface_create_similar_internal (abstract_surface, src_content, width, height, FALSE);
+
+    if (cairo_surface_status(new_surface))
+	return cairo_surface_status(new_surface);
+
+    _cairo_pattern_init_for_surface (&pattern.surface, src);
+
+    status = _cairo_surface_composite (CAIRO_OPERATOR_SOURCE,
+				       &pattern.base,
+				       NULL,
+				       new_surface,
+				       src_x, src_y,
+				       0, 0,
+				       0, 0,
+				       width, height);
+
+    _cairo_pattern_fini (&pattern.base);
+
+    if (status == CAIRO_STATUS_SUCCESS)
+	*clone_out = new_surface;
+    else
+	cairo_surface_destroy (new_surface);
+
+    return status;
+}
+
+
+cairo_status_t
 _cairo_win32_surface_finish (void *abstract_surface)
 {
     cairo_win32_surface_t *surface = abstract_surface;
@@ -443,16 +472,33 @@ _cairo_win32_surface_finish (void *abstract_surface)
     if (surface->image)
 	cairo_surface_destroy (surface->image);
 
-    if (surface->saved_clip)
-	DeleteObject (surface->saved_clip);
-
     /* If we created the Bitmap and DC, destroy them */
     if (surface->bitmap) {
 	SelectObject (surface->dc, surface->saved_dc_bitmap);
 	DeleteObject (surface->bitmap);
 	DeleteDC (surface->dc);
+    } else {
+	/* otherwise, restore the old clip region on the DC */
+	SelectClipRgn (surface->dc, surface->saved_clip);
+
+	if (surface->saved_clip == NULL) {
+	    /* We never had a clip region, so just restore the clip
+	     * to the bounds. */
+	    if (surface->clip_rect.width != 0 &&
+		surface->clip_rect.height != 0)
+	    {
+		IntersectClipRect (surface->dc,
+				   surface->clip_rect.x,
+				   surface->clip_rect.y,
+				   surface->clip_rect.x + surface->clip_rect.width,
+				   surface->clip_rect.y + surface->clip_rect.height);
+	    }
+	}
     }
 
+    if (surface->saved_clip)
+	DeleteObject (surface->saved_clip);
+
     return CAIRO_STATUS_SUCCESS;
 }
 
@@ -472,11 +518,12 @@ _cairo_win32_surface_get_subimage (cairo_win32_surface_t  *surface,
 	(cairo_win32_surface_t *) _cairo_win32_surface_create_similar_internal
 	(surface, content, width, height, TRUE);
     if (local->base.status)
-	return CAIRO_STATUS_NO_MEMORY;
+	return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 
     status = CAIRO_INT_STATUS_UNSUPPORTED;
 
-    if ((local->flags & CAIRO_WIN32_SURFACE_CAN_BITBLT) &&
+    /* Only BitBlt if the source surface supports it. */
+    if ((surface->flags & CAIRO_WIN32_SURFACE_CAN_BITBLT) &&
 	BitBlt (local->dc,
 		0, 0,
 		width, height,
@@ -547,9 +594,9 @@ _cairo_win32_surface_release_source_image (void                   *abstract_surf
 
 static cairo_status_t
 _cairo_win32_surface_acquire_dest_image (void                    *abstract_surface,
-					 cairo_rectangle_int16_t *interest_rect,
+					 cairo_rectangle_int_t   *interest_rect,
 					 cairo_image_surface_t  **image_out,
-					 cairo_rectangle_int16_t *image_rect,
+					 cairo_rectangle_int_t   *image_rect,
 					 void                   **image_extra)
 {
     cairo_win32_surface_t *surface = abstract_surface;
@@ -615,9 +662,9 @@ _cairo_win32_surface_acquire_dest_image (void                    *abstract_surfa
 
 static void
 _cairo_win32_surface_release_dest_image (void                    *abstract_surface,
-					 cairo_rectangle_int16_t *interest_rect,
+					 cairo_rectangle_int_t   *interest_rect,
 					 cairo_image_surface_t   *image,
-					 cairo_rectangle_int16_t *image_rect,
+					 cairo_rectangle_int_t   *image_rect,
 					 void                    *image_extra)
 {
     cairo_win32_surface_t *surface = abstract_surface;
@@ -736,7 +783,7 @@ static cairo_int_status_t
 _cairo_win32_surface_composite_inner (cairo_win32_surface_t *src,
 				      cairo_image_surface_t *src_image,
 				      cairo_win32_surface_t *dst,
-				      cairo_rectangle_int16_t src_extents,
+				      cairo_rectangle_int_t src_extents,
 				      cairo_rectangle_int32_t src_r,
 				      cairo_rectangle_int32_t dst_r,
 				      int alpha,
@@ -844,7 +891,7 @@ _cairo_win32_surface_composite (cairo_operator_t	op,
     cairo_image_surface_t *src_image = NULL;
 
     cairo_format_t src_format;
-    cairo_rectangle_int16_t src_extents;
+    cairo_rectangle_int_t src_extents;
 
     cairo_rectangle_int32_t src_r = { src_x, src_y, width, height };
     cairo_rectangle_int32_t dst_r = { dst_x, dst_y, width, height };
@@ -1312,7 +1359,7 @@ static cairo_int_status_t
 _cairo_win32_surface_fill_rectangles (void			*abstract_surface,
 				      cairo_operator_t		op,
 				      const cairo_color_t	*color,
-				      cairo_rectangle_int16_t		*rects,
+				      cairo_rectangle_int_t	*rects,
 				      int			num_rects)
 {
     cairo_win32_surface_t *surface = abstract_surface;
@@ -1374,11 +1421,11 @@ _cairo_win32_surface_fill_rectangles (void			*abstract_surface,
 }
 
 static cairo_int_status_t
-_cairo_win32_surface_set_clip_region (void              *abstract_surface,
-				      pixman_region16_t *region)
+_cairo_win32_surface_set_clip_region (void           *abstract_surface,
+				      cairo_region_t *region)
 {
     cairo_win32_surface_t *surface = abstract_surface;
-    cairo_status_t status;
+    cairo_status_t status = CAIRO_STATUS_SUCCESS;
 
     /* If we are in-memory, then we set the clip on the image surface
      * as well as on the underlying GDI surface.
@@ -1387,7 +1434,9 @@ _cairo_win32_surface_set_clip_region (void              *abstract_surface,
 	unsigned int serial;
 
 	serial = _cairo_surface_allocate_clip_serial (surface->image);
-	_cairo_surface_set_clip_region (surface->image, region, serial);
+	status = _cairo_surface_set_clip_region (surface->image, region, serial);
+	if (status)
+	    return status;
     }
 
     /* The semantics we want is that any clip set by cairo combines
@@ -1401,12 +1450,11 @@ _cairo_win32_surface_set_clip_region (void              *abstract_surface,
 	if (SelectClipRgn (surface->dc, surface->saved_clip) == ERROR)
 	    return _cairo_win32_print_gdi_error ("_cairo_win32_surface_set_clip_region (reset)");
 
-	return CAIRO_STATUS_SUCCESS;
-
+	status = CAIRO_STATUS_SUCCESS;
     } else {
-	pixman_box16_t *boxes = pixman_region_rects (region);
-	int num_boxes = pixman_region_num_rects (region);
-	pixman_box16_t *extents = pixman_region_extents (region);
+	cairo_rectangle_int_t extents;
+	cairo_box_int_t *boxes;
+	int num_boxes;
 	RGNDATA *data;
 	size_t data_size;
 	RECT *rects;
@@ -1415,57 +1463,82 @@ _cairo_win32_surface_set_clip_region (void              *abstract_surface,
 
 	/* Create a GDI region for the cairo region */
 
-	data_size = sizeof (RGNDATAHEADER) + num_boxes * sizeof (RECT);
-	data = malloc (data_size);
-	if (!data)
-	    return CAIRO_STATUS_NO_MEMORY;
-	rects = (RECT *)data->Buffer;
+	_cairo_region_get_extents (region, &extents);
+	status = _cairo_region_get_boxes (region, &num_boxes, &boxes);
+	if (status)
+	    return status;
 
-	data->rdh.dwSize = sizeof (RGNDATAHEADER);
-	data->rdh.iType = RDH_RECTANGLES;
-	data->rdh.nCount = num_boxes;
-	data->rdh.nRgnSize = num_boxes * sizeof (RECT);
-	data->rdh.rcBound.left = extents->x1;
-	data->rdh.rcBound.top = extents->y1;
-	data->rdh.rcBound.right = extents->x2;
-	data->rdh.rcBound.bottom = extents->y2;
+	if (num_boxes == 1 && 
+	    boxes[0].p1.x == 0 &&
+	    boxes[0].p1.y == 0 &&
+	    boxes[0].p2.x == surface->extents.width &&
+	    boxes[0].p2.y == surface->extents.height)
+	{
+	    gdi_region = NULL;
 
-	for (i = 0; i < num_boxes; i++) {
-	    rects[i].left = boxes[i].x1;
-	    rects[i].top = boxes[i].y1;
-	    rects[i].right = boxes[i].x2;
-	    rects[i].bottom = boxes[i].y2;
+	    SelectClipRgn (surface->dc, NULL);
+	    IntersectClipRect (surface->dc,
+			       boxes[0].p1.x,
+			       boxes[0].p1.y,
+			       boxes[0].p2.x,
+			       boxes[0].p2.y);
+
+	    _cairo_region_boxes_fini (region, boxes);
+	} else {
+	    data_size = sizeof (RGNDATAHEADER) + num_boxes * sizeof (RECT);
+	    data = malloc (data_size);
+	    if (!data) {
+		_cairo_region_boxes_fini (region, boxes);
+		return _cairo_error(CAIRO_STATUS_NO_MEMORY);
+	    }
+	    rects = (RECT *)data->Buffer;
+
+	    data->rdh.dwSize = sizeof (RGNDATAHEADER);
+	    data->rdh.iType = RDH_RECTANGLES;
+	    data->rdh.nCount = num_boxes;
+	    data->rdh.nRgnSize = num_boxes * sizeof (RECT);
+	    data->rdh.rcBound.left = extents.x;
+	    data->rdh.rcBound.top = extents.y;
+	    data->rdh.rcBound.right = extents.x + extents.width;
+	    data->rdh.rcBound.bottom = extents.y + extents.height;
+
+	    for (i = 0; i < num_boxes; i++) {
+		rects[i].left = boxes[i].p1.x;
+		rects[i].top = boxes[i].p1.y;
+		rects[i].right = boxes[i].p2.x;
+		rects[i].bottom = boxes[i].p2.y;
+	    }
+
+	    _cairo_region_boxes_fini (region, boxes);
+
+	    gdi_region = ExtCreateRegion (NULL, data_size, data);
+	    free (data);
+
+	    if (!gdi_region)
+		return _cairo_error (CAIRO_STATUS_NO_MEMORY);
+
+	    /* Combine the new region with the original clip */
+	    if (surface->saved_clip) {
+		if (CombineRgn (gdi_region, gdi_region, surface->saved_clip, RGN_AND) == ERROR)
+		    status = _cairo_win32_print_gdi_error ("_cairo_win32_surface_set_clip_region");
+	    }
+
+	    /* Then select the new clip region into our surface if everything went ok */
+	    if (status == CAIRO_STATUS_SUCCESS) {
+		if (SelectClipRgn (surface->dc, gdi_region) == ERROR)
+		    status = _cairo_win32_print_gdi_error ("_cairo_win32_surface_set_clip_region");
+	    }
+
+	    DeleteObject (gdi_region);
 	}
-
-	gdi_region = ExtCreateRegion (NULL, data_size, data);
-	free (data);
-
-	if (!gdi_region)
-	    return CAIRO_STATUS_NO_MEMORY;
-
-	/* Combine the new region with the original clip */
-
-	if (surface->saved_clip) {
-	    if (CombineRgn (gdi_region, gdi_region, surface->saved_clip, RGN_AND) == ERROR)
-		goto FAIL;
-	}
-
-	if (SelectClipRgn (surface->dc, gdi_region) == ERROR)
-	    goto FAIL;
-
-	DeleteObject (gdi_region);
-	return CAIRO_STATUS_SUCCESS;
-
-    FAIL:
-	status = _cairo_win32_print_gdi_error ("_cairo_win32_surface_set_clip_region");
-	DeleteObject (gdi_region);
-	return status;
     }
+
+    return status;
 }
 
-static cairo_int_status_t
+cairo_int_status_t
 _cairo_win32_surface_get_extents (void		          *abstract_surface,
-				  cairo_rectangle_int16_t *rectangle)
+				  cairo_rectangle_int_t   *rectangle)
 {
     cairo_win32_surface_t *surface = abstract_surface;
 
@@ -1482,7 +1555,7 @@ _cairo_win32_surface_flush (void *abstract_surface)
 
 #define STACK_GLYPH_SIZE 256
 
-static cairo_int_status_t
+cairo_int_status_t
 _cairo_win32_surface_show_glyphs (void			*surface,
 				  cairo_operator_t	 op,
 				  cairo_pattern_t	*source,
@@ -1525,8 +1598,10 @@ _cairo_win32_surface_show_glyphs (void			*surface,
 	return CAIRO_INT_STATUS_UNSUPPORTED;
 
     /* If we have a fallback mask clip set on the dst, we have
-     * to go through the fallback path */
-    if (dst->base.clip &&
+     * to go through the fallback path, but only if we're not
+     * doing this for printing */
+    if (dst->base.clip  &&
+	!(dst->flags & CAIRO_WIN32_SURFACE_FOR_PRINTING) &&
 	(dst->base.clip->mode != CAIRO_CLIP_MODE_REGION ||
 	 dst->base.clip->surface != NULL))
 	return CAIRO_INT_STATUS_UNSUPPORTED;
@@ -1546,8 +1621,8 @@ _cairo_win32_surface_show_glyphs (void			*surface,
     SetBkMode(dst->dc, TRANSPARENT);
 
     if (num_glyphs > STACK_GLYPH_SIZE) {
-	glyph_buf = (WORD *)malloc(num_glyphs * sizeof(WORD));
-        dxy_buf = (int *)malloc(num_glyphs * 2 * sizeof(int));
+	glyph_buf = (WORD *) _cairo_malloc_ab (num_glyphs, sizeof(WORD));
+        dxy_buf = (int *) _cairo_malloc_abc (num_glyphs, sizeof(int), 2);
     }
 
     /* It is vital that dx values for dxy_buf are calculated from the delta of
@@ -1637,13 +1712,15 @@ cairo_win32_surface_create (HDC hdc)
     RECT rect;
     int depth;
     cairo_format_t format;
+    int clipBoxType;
 
     /* Try to figure out the drawing bounds for the Device context
      */
-    if (GetClipBox (hdc, &rect) == ERROR) {
+    clipBoxType = GetClipBox (hdc, &rect);
+    if (clipBoxType == ERROR) {
 	_cairo_win32_print_gdi_error ("cairo_win32_surface_create");
 	/* XXX: Can we make a more reasonable guess at the error cause here? */
-	_cairo_error (CAIRO_STATUS_NO_MEMORY);
+	_cairo_error_throw (CAIRO_STATUS_NO_MEMORY);
 	return NIL_SURFACE;
     }
 
@@ -1661,7 +1738,7 @@ cairo_win32_surface_create (HDC hdc)
 	    format = CAIRO_FORMAT_A1;
 	else {
 	    _cairo_win32_print_gdi_error("cairo_win32_surface_create(bad BITSPIXEL)");
-	    _cairo_error (CAIRO_STATUS_NO_MEMORY);
+	    _cairo_error_throw (CAIRO_STATUS_NO_MEMORY);
 	    return NIL_SURFACE;
 	}
     } else {
@@ -1670,7 +1747,7 @@ cairo_win32_surface_create (HDC hdc)
 
     surface = malloc (sizeof (cairo_win32_surface_t));
     if (surface == NULL) {
-	_cairo_error (CAIRO_STATUS_NO_MEMORY);
+	_cairo_error_throw (CAIRO_STATUS_NO_MEMORY);
 	return NIL_SURFACE;
     }
 
@@ -1681,27 +1758,29 @@ cairo_win32_surface_create (HDC hdc)
     surface->bitmap = NULL;
     surface->is_dib = FALSE;
     surface->saved_dc_bitmap = NULL;
+    surface->brush = NULL;
+    surface->old_brush = NULL;
 
     surface->clip_rect.x = (int16_t) rect.left;
     surface->clip_rect.y = (int16_t) rect.top;
     surface->clip_rect.width = (uint16_t) (rect.right - rect.left);
     surface->clip_rect.height = (uint16_t) (rect.bottom - rect.top);
 
-    if (surface->clip_rect.width == 0 ||
-	surface->clip_rect.height == 0)
-    {
-	surface->saved_clip = NULL;
-    } else {
+    if (clipBoxType == COMPLEXREGION) {
 	surface->saved_clip = CreateRectRgn (0, 0, 0, 0);
 	if (GetClipRgn (hdc, surface->saved_clip) == 0) {
+	    /* this should never happen */
 	    DeleteObject(surface->saved_clip);
 	    surface->saved_clip = NULL;
 	}
+    } else {
+	surface->saved_clip = NULL;
     }
 
     surface->extents = surface->clip_rect;
 
     surface->flags = _cairo_win32_flags_for_dc (surface->dc);
+    surface->clip_saved_dc = 0;
 
     _cairo_surface_init (&surface->base, &cairo_win32_surface_backend,
 			 _cairo_content_from_format (format));
@@ -1755,11 +1834,10 @@ cairo_win32_surface_create_with_ddb (HDC hdc,
     cairo_win32_surface_t *new_surf;
     HBITMAP ddb;
     HDC screen_dc, ddb_dc;
-    HRGN crgn;
     HBITMAP saved_dc_bitmap;
 
     if (format != CAIRO_FORMAT_RGB24)
-	return NULL;
+	return NIL_SURFACE;
 /* XXX handle these eventually
 	format != CAIRO_FORMAT_A8 ||
 	format != CAIRO_FORMAT_A1)
@@ -1772,20 +1850,34 @@ cairo_win32_surface_create_with_ddb (HDC hdc,
 	screen_dc = NULL;
     }
 
-    ddb = CreateCompatibleBitmap (hdc, width, height);
     ddb_dc = CreateCompatibleDC (hdc);
+    if (ddb_dc == NULL) {
+	_cairo_win32_print_gdi_error("CreateCompatibleDC");
+	new_surf = (cairo_win32_surface_t*) NIL_SURFACE;
+	goto FINISH;
+    }
+
+    ddb = CreateCompatibleBitmap (hdc, width, height);
+    if (ddb == NULL) {
+	DeleteDC (ddb_dc);
+
+	/* Note that if an app actually does hit this out of memory
+	 * condition, it's going to have lots of other issues, as
+	 * video memory is probably exhausted.
+	 */
+	_cairo_win32_print_gdi_error("CreateCompatibleBitmap");
+	new_surf = (cairo_win32_surface_t*) NIL_SURFACE;
+	goto FINISH;
+    }
 
     saved_dc_bitmap = SelectObject (ddb_dc, ddb);
 
-    crgn = CreateRectRgn (0, 0, width, height);
-    SelectClipRgn (ddb_dc, crgn);
-    DeleteObject (crgn);
-
     new_surf = (cairo_win32_surface_t*) cairo_win32_surface_create (ddb_dc);
     new_surf->bitmap = ddb;
     new_surf->saved_dc_bitmap = saved_dc_bitmap;
     new_surf->is_dib = FALSE;
 
+FINISH:
     if (screen_dc)
 	ReleaseDC (NULL, screen_dc);
 
@@ -1796,7 +1888,9 @@ cairo_win32_surface_create_with_ddb (HDC hdc,
  * _cairo_surface_is_win32:
  * @surface: a #cairo_surface_t
  *
- * Checks if a surface is an #cairo_win32_surface_t
+ * Checks if a surface is a win32 surface.  This will
+ * return False if this is a win32 printing surface; use
+ * _cairo_surface_is_win32_printing() to check for that.
  *
  * Return value: True if the surface is an win32 surface
  **/
@@ -1822,10 +1916,8 @@ cairo_win32_surface_get_dc (cairo_surface_t *surface)
 {
     cairo_win32_surface_t *winsurf;
 
-    if (surface == NULL)
-	return NULL;
-
-    if (!_cairo_surface_is_win32(surface))
+    if (!_cairo_surface_is_win32(surface) &&
+	!_cairo_surface_is_win32_printing(surface))
 	return NULL;
 
     winsurf = (cairo_win32_surface_t *) surface;
@@ -1887,7 +1979,7 @@ static const cairo_surface_backend_t cairo_win32_surface_backend = {
     _cairo_win32_surface_release_source_image,
     _cairo_win32_surface_acquire_dest_image,
     _cairo_win32_surface_release_dest_image,
-    NULL, /* clone_similar */
+    _cairo_win32_surface_clone_similar,
     _cairo_win32_surface_composite,
     _cairo_win32_surface_fill_rectangles,
     NULL, /* composite_trapezoids */
