@@ -297,14 +297,14 @@ _CAIRO_MASK_FORMAT (cairo_format_masks_t *masks, cairo_format_t *format)
 
 static cairo_status_t
 _get_image_surface (cairo_xcb_surface_t     *surface,
-		    cairo_rectangle_int16_t *interest_rect,
+		    cairo_rectangle_int_t   *interest_rect,
 		    cairo_image_surface_t  **image_out,
-		    cairo_rectangle_int16_t *image_rect)
+		    cairo_rectangle_int_t   *image_rect)
 {
     cairo_image_surface_t *image;
     xcb_get_image_reply_t *imagerep;
     int bpp, bytes_per_line;
-    int x1, y1, x2, y2;
+    short x1, y1, x2, y2;
     unsigned char *data;
     cairo_format_masks_t masks;
     cairo_format_t format;
@@ -398,15 +398,15 @@ _get_image_surface (cairo_xcb_surface_t     *surface,
 
     }
     if (!imagerep)
-	return CAIRO_STATUS_NO_MEMORY;
+	return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 
     bpp = _bits_per_pixel(surface->dpy, imagerep->depth);
     bytes_per_line = _bytes_per_line(surface->dpy, surface->width, bpp);
 
-    data = malloc (bytes_per_line * surface->height);
+    data = _cairo_malloc_ab (surface->height, bytes_per_line);
     if (data == NULL) {
 	free (imagerep);
-	return CAIRO_STATUS_NO_MEMORY;
+	return _cairo_error (CAIRO_STATUS_NO_MEMORY);
     }
 
     memcpy (data, xcb_get_image_data (imagerep), bytes_per_line * surface->height);
@@ -479,7 +479,7 @@ _get_image_surface (cairo_xcb_surface_t     *surface,
 
  FAIL:
     free (data);
-    return CAIRO_STATUS_NO_MEMORY;
+    return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 }
 
 static void
@@ -591,7 +591,8 @@ _draw_image_surface (cairo_xcb_surface_t    *surface,
 	data_len = height * data_bpl;
 	data_line = data = malloc(data_len);
 	if (data == NULL)
-	    return CAIRO_STATUS_NO_MEMORY;
+	    return _cairo_error (CAIRO_STATUS_NO_MEMORY);
+
 	image_line = image->data + src_y * bpl + (src_x * bpp / 8);
 	while (line++ < height) {
 	    memcpy(data_line, image_line, data_bpl);
@@ -642,9 +643,9 @@ _cairo_xcb_surface_release_source_image (void                   *abstract_surfac
 
 static cairo_status_t
 _cairo_xcb_surface_acquire_dest_image (void                    *abstract_surface,
-				       cairo_rectangle_int16_t *interest_rect,
+				       cairo_rectangle_int_t   *interest_rect,
 				       cairo_image_surface_t  **image_out,
-				       cairo_rectangle_int16_t *image_rect_out,
+				       cairo_rectangle_int_t   *image_rect_out,
 				       void                   **image_extra)
 {
     cairo_xcb_surface_t *surface = abstract_surface;
@@ -663,9 +664,9 @@ _cairo_xcb_surface_acquire_dest_image (void                    *abstract_surface
 
 static void
 _cairo_xcb_surface_release_dest_image (void                   *abstract_surface,
-				       cairo_rectangle_int16_t      *interest_rect,
+				       cairo_rectangle_int_t  *interest_rect,
 				       cairo_image_surface_t  *image,
-				       cairo_rectangle_int16_t      *image_rect,
+				       cairo_rectangle_int_t  *image_rect,
 				       void                   *image_extra)
 {
     cairo_xcb_surface_t *surface = abstract_surface;
@@ -720,7 +721,7 @@ _cairo_xcb_surface_clone_similar (void			*abstract_surface,
 	    _cairo_xcb_surface_create_similar (surface, content,
 					       image_src->width, image_src->height);
 	if (clone->base.status)
-	    return CAIRO_STATUS_NO_MEMORY;
+	    return clone->base.status;
 
 	_draw_image_surface (clone, image_src, src_x, src_y,
 			     width, height, src_x, src_y);
@@ -742,17 +743,17 @@ _cairo_xcb_surface_set_matrix (cairo_xcb_surface_t *surface,
     if (!surface->src_picture)
 	return CAIRO_STATUS_SUCCESS;
 
-    xtransform.matrix11 = _cairo_fixed_from_double (matrix->xx);
-    xtransform.matrix12 = _cairo_fixed_from_double (matrix->xy);
-    xtransform.matrix13 = _cairo_fixed_from_double (matrix->x0);
+    xtransform.matrix11 = _cairo_fixed_16_16_from_double (matrix->xx);
+    xtransform.matrix12 = _cairo_fixed_16_16_from_double (matrix->xy);
+    xtransform.matrix13 = _cairo_fixed_16_16_from_double (matrix->x0);
 
-    xtransform.matrix21 = _cairo_fixed_from_double (matrix->yx);
-    xtransform.matrix22 = _cairo_fixed_from_double (matrix->yy);
-    xtransform.matrix23 = _cairo_fixed_from_double (matrix->y0);
+    xtransform.matrix21 = _cairo_fixed_16_16_from_double (matrix->yx);
+    xtransform.matrix22 = _cairo_fixed_16_16_from_double (matrix->yy);
+    xtransform.matrix23 = _cairo_fixed_16_16_from_double (matrix->y0);
 
     xtransform.matrix31 = 0;
     xtransform.matrix32 = 0;
-    xtransform.matrix33 = _cairo_fixed_from_double (1);
+    xtransform.matrix33 = 1 << 16;
 
     if (!CAIRO_SURFACE_RENDER_HAS_PICTURE_TRANSFORM (surface))
     {
@@ -1246,11 +1247,14 @@ static cairo_int_status_t
 _cairo_xcb_surface_fill_rectangles (void			     *abstract_surface,
 				     cairo_operator_t	      op,
 				     const cairo_color_t	*     color,
-				     cairo_rectangle_int16_t *rects,
+				     cairo_rectangle_int_t *rects,
 				     int			      num_rects)
 {
     cairo_xcb_surface_t *surface = abstract_surface;
     xcb_render_color_t render_color;
+    xcb_rectangle_t static_xrects[16];
+    xcb_rectangle_t *xrects = static_xrects;
+    int i;
 
     if (!CAIRO_SURFACE_RENDER_HAS_FILL_RECTANGLE (surface))
 	return CAIRO_INT_STATUS_UNSUPPORTED;
@@ -1260,12 +1264,27 @@ _cairo_xcb_surface_fill_rectangles (void			     *abstract_surface,
     render_color.blue  = color->blue_short;
     render_color.alpha = color->alpha_short;
 
-    /* XXX: This xcb_rectangle_t cast is evil... it needs to go away somehow. */
+    if (num_rects > ARRAY_LENGTH(static_xrects)) {
+        xrects = _cairo_malloc_ab (num_rects, sizeof(xcb_rectangle_t));
+	if (xrects == NULL)
+	    return _cairo_error (CAIRO_STATUS_NO_MEMORY);
+    }
+
+    for (i = 0; i < num_rects; i++) {
+        xrects[i].x = rects[i].x;
+        xrects[i].y = rects[i].y;
+        xrects[i].width = rects[i].width;
+        xrects[i].height = rects[i].height;
+    }
+
     _cairo_xcb_surface_ensure_dst_picture (surface);
     xcb_render_fill_rectangles (surface->dpy,
 			   _render_operator (op),
 			   surface->dst_picture,
-			   render_color, num_rects, (xcb_rectangle_t *) rects);
+			   render_color, num_rects, xrects);
+
+    if (xrects != static_xrects)
+        free(xrects);
 
     return CAIRO_STATUS_SUCCESS;
 }
@@ -1328,21 +1347,23 @@ _create_trapezoid_mask (cairo_xcb_surface_t *dst,
     mask_picture = _create_a8_picture (dst, &transparent, width, height, FALSE);
     solid_picture = _create_a8_picture (dst, &solid, width, height, TRUE);
 
-    offset_traps = malloc (sizeof (xcb_render_trapezoid_t) * num_traps);
-    if (!offset_traps)
+    offset_traps = _cairo_malloc_ab (num_traps, sizeof (xcb_render_trapezoid_t));
+    if (!offset_traps) {
+	_cairo_error (CAIRO_STATUS_NO_MEMORY);
 	return XCB_NONE;
+    }
 
     for (i = 0; i < num_traps; i++) {
-	offset_traps[i].top = traps[i].top - 0x10000 * dst_y;
-	offset_traps[i].bottom = traps[i].bottom - 0x10000 * dst_y;
-	offset_traps[i].left.p1.x = traps[i].left.p1.x - 0x10000 * dst_x;
-	offset_traps[i].left.p1.y = traps[i].left.p1.y - 0x10000 * dst_y;
-	offset_traps[i].left.p2.x = traps[i].left.p2.x - 0x10000 * dst_x;
-	offset_traps[i].left.p2.y = traps[i].left.p2.y - 0x10000 * dst_y;
-	offset_traps[i].right.p1.x = traps[i].right.p1.x - 0x10000 * dst_x;
-	offset_traps[i].right.p1.y = traps[i].right.p1.y - 0x10000 * dst_y;
-	offset_traps[i].right.p2.x = traps[i].right.p2.x - 0x10000 * dst_x;
-	offset_traps[i].right.p2.y = traps[i].right.p2.y - 0x10000 * dst_y;
+	offset_traps[i].top = _cairo_fixed_to_16_16(traps[i].top) - 0x10000 * dst_y;
+	offset_traps[i].bottom = _cairo_fixed_to_16_16(traps[i].bottom) - 0x10000 * dst_y;
+	offset_traps[i].left.p1.x = _cairo_fixed_to_16_16(traps[i].left.p1.x) - 0x10000 * dst_x;
+	offset_traps[i].left.p1.y = _cairo_fixed_to_16_16(traps[i].left.p1.y) - 0x10000 * dst_y;
+	offset_traps[i].left.p2.x = _cairo_fixed_to_16_16(traps[i].left.p2.x) - 0x10000 * dst_x;
+	offset_traps[i].left.p2.y = _cairo_fixed_to_16_16(traps[i].left.p2.y) - 0x10000 * dst_y;
+	offset_traps[i].right.p1.x = _cairo_fixed_to_16_16(traps[i].right.p1.x) - 0x10000 * dst_x;
+	offset_traps[i].right.p1.y = _cairo_fixed_to_16_16(traps[i].right.p1.y) - 0x10000 * dst_y;
+	offset_traps[i].right.p2.x = _cairo_fixed_to_16_16(traps[i].right.p2.x) - 0x10000 * dst_x;
+        offset_traps[i].right.p2.y = _cairo_fixed_to_16_16(traps[i].right.p2.y) - 0x10000 * dst_y;
     }
 
     xcb_render_trapezoids (dst->dpy, XCB_RENDER_PICT_OP_ADD,
@@ -1445,7 +1466,7 @@ _cairo_xcb_surface_composite_trapezoids (cairo_operator_t	op,
 						       dst_x, dst_y, width, height,
 						       render_format);
 	if (!mask_picture) {
-	    status = CAIRO_STATUS_NO_MEMORY;
+	    status = _cairo_error (CAIRO_STATUS_NO_MEMORY);
 	    goto BAIL;
 	}
 
@@ -1470,14 +1491,41 @@ _cairo_xcb_surface_composite_trapezoids (cairo_operator_t	op,
 								 dst_x, dst_y, width, height);
 
     } else {
-	/* XXX: The XTrapezoid cast is evil and needs to go away somehow. */
+        xcb_render_trapezoid_t xtraps_stack[16];
+        xcb_render_trapezoid_t *xtraps = xtraps_stack;
+        int i;
+
+        if (num_traps > ARRAY_LENGTH(xtraps_stack)) {
+            xtraps = _cairo_malloc_ab (num_traps, sizeof(xcb_render_trapezoid_t));
+            if (xtraps == NULL) {
+                status = _cairo_error (CAIRO_STATUS_NO_MEMORY);
+                goto BAIL;
+            }
+        }
+
+        for (i = 0; i < num_traps; i++) {
+            xtraps[i].top = _cairo_fixed_to_16_16(traps[i].top);
+            xtraps[i].bottom = _cairo_fixed_to_16_16(traps[i].bottom);
+            xtraps[i].left.p1.x = _cairo_fixed_to_16_16(traps[i].left.p1.x);
+            xtraps[i].left.p1.y = _cairo_fixed_to_16_16(traps[i].left.p1.y);
+            xtraps[i].left.p2.x = _cairo_fixed_to_16_16(traps[i].left.p2.x);
+            xtraps[i].left.p2.y = _cairo_fixed_to_16_16(traps[i].left.p2.y);
+            xtraps[i].right.p1.x = _cairo_fixed_to_16_16(traps[i].right.p1.x);
+            xtraps[i].right.p1.y = _cairo_fixed_to_16_16(traps[i].right.p1.y);
+            xtraps[i].right.p2.x = _cairo_fixed_to_16_16(traps[i].right.p2.x);
+            xtraps[i].right.p2.y = _cairo_fixed_to_16_16(traps[i].right.p2.y);
+        }
+
 	xcb_render_trapezoids (dst->dpy,
 				    _render_operator (op),
 				    src->src_picture, dst->dst_picture,
 				    render_format->id,
 				    render_src_x + attributes.x_offset,
 				    render_src_y + attributes.y_offset,
-				    num_traps, (xcb_render_trapezoid_t *) traps);
+				    num_traps, xtraps);
+
+        if (xtraps != xtraps_stack)
+            free(xtraps);
     }
 
  BAIL:
@@ -1487,8 +1535,8 @@ _cairo_xcb_surface_composite_trapezoids (cairo_operator_t	op,
 }
 
 static cairo_int_status_t
-_cairo_xcb_surface_set_clip_region (void              *abstract_surface,
-				    pixman_region16_t *region)
+_cairo_xcb_surface_set_clip_region (void           *abstract_surface,
+				    cairo_region_t *region)
 {
     cairo_xcb_surface_t *surface = abstract_surface;
 
@@ -1509,28 +1557,34 @@ _cairo_xcb_surface_set_clip_region (void              *abstract_surface,
 	    xcb_render_change_picture (surface->dpy, surface->dst_picture,
 		XCB_RENDER_CP_CLIP_MASK, none);
     } else {
-	pixman_box16_t *boxes;
+	cairo_box_int_t *boxes;
+	cairo_status_t status;
 	xcb_rectangle_t *rects = NULL;
 	int n_boxes, i;
 
-	n_boxes = pixman_region_num_rects (region);
+	status = _cairo_region_get_boxes (region, &n_boxes, &boxes);
+        if (status)
+            return status;
+
 	if (n_boxes > 0) {
-	    rects = malloc (sizeof(xcb_rectangle_t) * n_boxes);
-	    if (rects == NULL)
-		return CAIRO_STATUS_NO_MEMORY;
+	    rects = _cairo_malloc_ab (n_boxes, sizeof(xcb_rectangle_t));
+	    if (rects == NULL) {
+                _cairo_region_boxes_fini (region, boxes);
+		return _cairo_error (CAIRO_STATUS_NO_MEMORY);
+            }
 	} else {
 	    rects = NULL;
 	}
 
-	boxes = pixman_region_rects (region);
-
 	for (i = 0; i < n_boxes; i++) {
-	    rects[i].x = boxes[i].x1;
-	    rects[i].y = boxes[i].y1;
-	    rects[i].width = boxes[i].x2 - boxes[i].x1;
-	    rects[i].height = boxes[i].y2 - boxes[i].y1;
+	    rects[i].x = boxes[i].p1.x;
+	    rects[i].y = boxes[i].p1.y;
+	    rects[i].width = boxes[i].p2.x - boxes[i].p1.x;
+	    rects[i].height = boxes[i].p2.y - boxes[i].p1.y;
 	}
 
+        _cairo_region_boxes_fini (region, boxes);
+
 	surface->have_clip_rects = TRUE;
 	surface->clip_rects = rects;
 	surface->num_clip_rects = n_boxes;
@@ -1547,7 +1601,7 @@ _cairo_xcb_surface_set_clip_region (void              *abstract_surface,
 
 static cairo_int_status_t
 _cairo_xcb_surface_get_extents (void		        *abstract_surface,
-				cairo_rectangle_int16_t *rectangle)
+				cairo_rectangle_int_t   *rectangle)
 {
     cairo_xcb_surface_t *surface = abstract_surface;
 
@@ -1941,7 +1995,7 @@ _cairo_xcb_surface_font_init (xcb_connection_t		    *dpy,
 
     font_private = malloc (sizeof (cairo_xcb_surface_font_private_t));
     if (!font_private)
-	return CAIRO_STATUS_NO_MEMORY;
+	return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 
     font_private->dpy = dpy;
     font_private->format = format;
@@ -2038,45 +2092,9 @@ _cairo_xcb_surface_add_glyph (xcb_connection_t *dpy,
 	    goto BAIL;
     }
 
-    /*
-     *  Most of the font rendering system thinks of glyph tiles as having
-     *  an origin at (0,0) and an x and y bounding box "offset" which
-     *  extends possibly off into negative coordinates, like so:
-     *
-     *
-     *       (x,y) <-- probably negative numbers
-     *         +----------------+
-     *         |      .         |
-     *         |      .         |
-     *         |......(0,0)     |
-     *         |                |
-     *         |                |
-     *         +----------------+
-     *                  (width+x,height+y)
-     *
-     *  This is a postscript-y model, where each glyph has its own
-     *  coordinate space, so it's what we expose in terms of metrics. It's
-     *  apparently what everyone's expecting. Everyone except the Render
-     *  extension. Render wants to see a glyph tile starting at (0,0), with
-     *  an origin offset inside, like this:
-     *
-     *       (0,0)
-     *         +---------------+
-     *         |      .        |
-     *         |      .        |
-     *         |......(x,y)    |
-     *         |               |
-     *         |               |
-     *         +---------------+
-     *                   (width,height)
-     *
-     *  Luckily, this is just the negation of the numbers we already have
-     *  sitting around for x and y.
-     */
-
     /* XXX: FRAGILE: We're ignore device_transform scaling here. A bug? */
-    glyph_info.x = - _cairo_lround (glyph_surface->base.device_transform.x0);
-    glyph_info.y = - _cairo_lround (glyph_surface->base.device_transform.y0);
+    glyph_info.x = _cairo_lround (glyph_surface->base.device_transform.x0);
+    glyph_info.y = _cairo_lround (glyph_surface->base.device_transform.y0);
     glyph_info.width = glyph_surface->width;
     glyph_info.height = glyph_surface->height;
     glyph_info.x_off = 0;
@@ -2095,7 +2113,7 @@ _cairo_xcb_surface_add_glyph (xcb_connection_t *dpy,
 
 	    new = malloc (c);
 	    if (!new) {
-		status = CAIRO_STATUS_NO_MEMORY;
+		status = _cairo_error (CAIRO_STATUS_NO_MEMORY);
 		goto BAIL;
 	    }
 	    n = new;
@@ -2121,7 +2139,7 @@ _cairo_xcb_surface_add_glyph (xcb_connection_t *dpy,
 
 	    new = malloc (c);
 	    if (new == NULL) {
-		status = CAIRO_STATUS_NO_MEMORY;
+		status = _cairo_error (CAIRO_STATUS_NO_MEMORY);
 		goto BAIL;
 	    }
 	    n = new;
@@ -2360,9 +2378,9 @@ _cairo_xcb_surface_show_glyphs (void                *abstract_dst,
     /* We make a copy of the glyphs so that we can elide any size-zero
      * glyphs to workaround an X server bug, (present in at least Xorg
      * 7.1 without EXA). */
-    output_glyphs = malloc (num_glyphs * sizeof (cairo_glyph_t));
+    output_glyphs = _cairo_malloc_ab (num_glyphs, sizeof (cairo_glyph_t));
     if (output_glyphs == NULL)
-	return CAIRO_STATUS_NO_MEMORY;
+	return _cairo_error (CAIRO_STATUS_NO_MEMORY);
 
     /* After passing all those tests, we're now committed to rendering
      * these glyphs or to fail trying. We first upload any glyphs to
